# -*- coding: utf-8 -*-

# Copyright (C) 2012-2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

from __future__ import absolute_import
from __future__ import unicode_literals

import binascii
import itertools
import re

import hawkey
import libdnf.transaction
import rpm

import dnf
import dnf.exceptions
import dnf.package
import dnf.subject
import dnf.transaction

import tests.support
from tests.support import mock


class BaseTest(tests.support.TestCase):

    @staticmethod
    def _setup_packages(history):
        pkg = tests.support.MockPackage('pepper-20-0.x86_64')
        pkg._force_swdb_repoid = "main"
        history.rpm.add_install(pkg)
        history.beg("", [], [])
        for tsi in history._swdb.getItems():
            if tsi.getState() == libdnf.transaction.TransactionItemState_UNKNOWN:
                tsi.setState(libdnf.transaction.TransactionItemState_DONE)
        history.end("")
        history.close()

    def test_instance(self):
        base = tests.support.MockBase()
        self.assertIsNotNone(base)
        base.close()

    @mock.patch('dnf.rpm.detect_releasever', lambda x: 'x')
    @mock.patch('dnf.util.am_i_root', lambda: True)
    def test_default_config_root(self):
        base = dnf.Base()
        self.assertIsNotNone(base.conf)
        self.assertIsNotNone(base.conf.cachedir)
        reg = re.compile('/var/cache/dnf')
        self.assertIsNotNone(reg.match(base.conf.cachedir))
        base.close()

    @mock.patch('dnf.rpm.detect_releasever', lambda x: 'x')
    @mock.patch('dnf.util.am_i_root', lambda: False)
    def test_default_config_user(self):
        base = dnf.Base()
        self.assertIsNotNone(base.conf)
        self.assertIsNotNone(base.conf.cachedir)
        reg = re.compile('/var/tmp/dnf-[a-zA-Z0-9_-]+')
        self.assertIsNotNone(reg.match(base.conf.cachedir))
        base.close()

    def test_reset(self):
        base = tests.support.MockBase('main')
        base.reset(sack=True, repos=False)
        self.assertIsNone(base._sack)
        self.assertLength(base.repos, 1)
        base.close()

    @mock.patch('dnf.rpm.transaction.TransactionWrapper')
    def test_ts(self, mock_ts):
        base = dnf.Base()
        self.assertEqual(base._priv_ts, None)
        ts = base._ts
        # check the setup is correct
        ts.setFlags.call_args.assert_called_with(0)
        flags = ts.setProbFilter.call_args[0][0]
        self.assertTrue(flags & rpm.RPMPROB_FILTER_OLDPACKAGE)
        self.assertFalse(flags & rpm.RPMPROB_FILTER_REPLACEPKG)
        # check file conflicts are reported:
        self.assertFalse(flags & rpm.RPMPROB_FILTER_REPLACENEWFILES)
        # check we can close the connection
        del base._ts
        self.assertEqual(base._priv_ts, None)
        ts.close.assert_called_once_with()
        base.close()

    def test_iter_userinstalled(self):
        """Test iter_userinstalled with a package installed by the user."""
        base = tests.support.MockBase()
        self._setup_packages(base.history)
        base._sack = tests.support.mock_sack('main')
        pkg, = base.sack.query().installed().filter(name='pepper')
        # reason and repo are set in _setup_packages() already
        self.assertEqual(base.history.user_installed(pkg), True)
        self.assertEqual(base.history.repo(pkg), 'main')
        base.close()

    def test_iter_userinstalled_badfromrepo(self):
        """Test iter_userinstalled with a package installed from a bad repository."""
        base = tests.support.MockBase()
        base._sack = tests.support.mock_sack('main')
        self._setup_packages(base.history)

        history = base.history
        pkg = tests.support.MockPackage('pepper-20-0.x86_64')
        pkg._force_swdb_repoid = "anakonda"
        history.rpm.add_install(pkg)
        history.beg("", [], [])
        for tsi in history._swdb.getItems():
            if tsi.getState() == libdnf.transaction.TransactionItemState_UNKNOWN:
                tsi.setState(libdnf.transaction.TransactionItemState_DONE)
        history.end("")
        history.close()

        pkg, = base.sack.query().installed().filter(name='pepper')
        self.assertEqual(base.history.user_installed(pkg), True)
        self.assertEqual(base.history.repo(pkg), 'anakonda')
        base.close()

    def test_iter_userinstalled_badreason(self):
        """Test iter_userinstalled with a package installed for a wrong reason."""
        base = tests.support.MockBase()
        base._sack = tests.support.mock_sack('main')
        self._setup_packages(base.history)

        history = base.history
        pkg = tests.support.MockPackage('pepper-20-0.x86_64')
        pkg._force_swdb_repoid = "main"
        history.rpm.add_install(pkg, reason=libdnf.transaction.TransactionItemReason_DEPENDENCY)
        history.beg("", [], [])
        for tsi in history._swdb.getItems():
            if tsi.getState() == libdnf.transaction.TransactionItemState_UNKNOWN:
                tsi.setState(libdnf.transaction.TransactionItemState_DONE)
        history.end("")
        history.close()

        pkg, = base.sack.query().installed().filter(name='pepper')
        self.assertEqual(base.history.user_installed(pkg), False)
        self.assertEqual(base.history.repo(pkg), 'main')
        base.close()


class MockBaseTest(tests.support.DnfBaseTestCase):
    """Test the Base methods that need a Sack."""

    REPOS = ["main"]
    INIT_SACK = True

    def test_add_remote_rpms(self):
        pkgs = self.base.add_remote_rpms([tests.support.TOUR_50_PKG_PATH])
        self.assertIsInstance(pkgs[0], dnf.package.Package)
        self.assertEqual(pkgs[0].name, 'tour')


class BuildTransactionTest(tests.support.DnfBaseTestCase):

    REPOS = ["updates"]

    def test_resolve(self):
        self.base.upgrade("pepper")
        self.assertTrue(self.base.resolve())
        self.base._ds_callback.assert_has_calls([
            mock.call.start(),
            mock.call.pkg_added(mock.ANY, 'ud'),
            mock.call.pkg_added(mock.ANY, 'u')
        ])
        self.assertLength(self.base.transaction, 2)


# verify transaction test helpers
HASH = "68e9ded8ea25137c964a638f12e9987c"


def mock_sack_fn():
    return (lambda base: tests.support.TestSack(tests.support.REPO_DIR, base))


@property
def ret_pkgid(self):
    return self.name


class VerifyTransactionTest(tests.support.DnfBaseTestCase):

    REPOS = ["main"]
    INIT_TRANSACTION = True

    @mock.patch('dnf.sack._build_sack', new_callable=mock_sack_fn)
    @mock.patch('dnf.package.Package._pkgid', ret_pkgid)  # neutralize @property
    def test_verify_transaction(self, unused_build_sack):
        # we don't simulate the transaction itself here, just "install" what is
        # already there and "remove" what is not.

        tsis = []

        new_pkg = self.base.sack.query().available().filter(name="pepper")[1]
        new_pkg._chksum = (hawkey.CHKSUM_MD5, binascii.unhexlify(HASH))
        new_pkg.repo = mock.Mock()
        new_pkg._force_swdb_repoid = "main"
        self.history.rpm.add_install(new_pkg)

        removed_pkg = self.base.sack.query().available().filter(name="mrkite")[0]
        removed_pkg._force_swdb_repoid = "main"
        self.history.rpm.add_remove(removed_pkg)

        self._swdb_commit(tsis)
        self.base._verify_transaction()

        pkg = self.base.history.package_data(new_pkg)
        self.assertEqual(pkg.ui_from_repo(), '@main')
        self.assertEqual(pkg.action_name, "Install")
        self.assertEqual(pkg.get_reason(), libdnf.transaction.TransactionItemReason_USER)


class InstallReasonTest(tests.support.ResultTestCase):

    REPOS = ["main"]

    def test_reason(self):
        self.base.install("mrkite")
        self.base.resolve()
        new_pkgs = self.base._transaction._get_items(dnf.transaction.PKG_INSTALL)
        pkg_reasons = [(tsi.name, tsi.reason) for tsi in new_pkgs]
        self.assertCountEqual([
            ("mrkite", libdnf.transaction.TransactionItemReason_USER),
            ("trampoline", libdnf.transaction.TransactionItemReason_DEPENDENCY)],
            pkg_reasons
        )


class InstalledMatchingTest(tests.support.ResultTestCase):

    REPOS = ["main"]

    def test_query_matching(self):
        subj = dnf.subject.Subject("pepper")
        query = subj.get_best_query(self.sack)
        inst, avail = self.base._query_matches_installed(query)
        self.assertCountEqual(['pepper-20-0.x86_64'], map(str, inst))
        self.assertCountEqual(['pepper-20-0.src'], map(str, itertools.chain.from_iterable(avail)))

    def test_selector_matching(self):
        subj = dnf.subject.Subject("pepper")
        sltr = subj.get_best_selector(self.sack)
        inst = self.base._sltr_matches_installed(sltr)
        self.assertCountEqual(['pepper-20-0.x86_64'], map(str, inst))


class CompsTest(tests.support.DnfBaseTestCase):
    # Also see test_comps.py

    REPOS = ["main"]
    COMPS = True

    # prevent creating the gen/ directory:
    @mock.patch('dnf.yum.misc.repo_gen_decompress', lambda x, y: x)
    def test_read_comps(self):
        self.assertLength(self.base.comps.groups, tests.support.TOTAL_GROUPS)

    def test_read_comps_disabled(self):
        self.base.repos['main'].enablegroups = False
        self.assertEmpty(self.base.read_comps())


class Goal2TransactionTest(tests.support.DnfBaseTestCase):

    REPOS = ["main", "updates"]

    def test_upgrade(self):
        self.base.upgrade("hole")
        self.assertTrue(self.base._run_hawkey_goal(self.goal, allow_erasing=False))
        ts = self.base._goal2transaction(self.goal)
        self.assertLength(ts, 3)
        tsis = list(ts)

        tsi = tsis[0]
        self.assertEqual(str(tsi.pkg), "hole-2-1.x86_64")
        self.assertEqual(tsi.action, libdnf.transaction.TransactionItemAction_UPGRADE)

        tsi = tsis[1]
        self.assertEqual(str(tsi.pkg), "hole-1-1.x86_64")
        self.assertEqual(tsi.action, libdnf.transaction.TransactionItemAction_UPGRADED)

        tsi = tsis[2]
        self.assertEqual(str(tsi.pkg), "tour-5-0.noarch")
        self.assertEqual(tsi.action, libdnf.transaction.TransactionItemAction_OBSOLETED)



# -*- coding: utf-8 -*-


from __future__ import absolute_import
from __future__ import unicode_literals

import dnf
import dnf.cli.demand

from .common import TestCase


class DnfCliDemandApiTest(TestCase):
    def setUp(self):
        self.demand_sheet = dnf.cli.demand.DemandSheet()

    def test_demand_sheet(self):
        # dnf.cli.demand.DemandSheet
        self.assertHasAttr(dnf.cli.demand, "DemandSheet")
        self.assertHasType(dnf.cli.demand.DemandSheet, object)

    def test_init(self):
        _ = dnf.cli.demand.DemandSheet()

    def test_allow_erasing(self):
        # dnf.cli.demand.DemandSheet.allow_erasing
        self.assertHasAttr(self.demand_sheet, "allow_erasing")
        self.assertHasType(self.demand_sheet.allow_erasing, bool)

    def test_available_repos(self):
        # dnf.cli.demand.DemandSheet.available_repos
        self.assertHasAttr(self.demand_sheet, "available_repos")
        self.assertHasType(self.demand_sheet.available_repos, bool)

    def test_resolving(self):
        # dnf.cli.demand.DemandSheet.resolving
        self.assertHasAttr(self.demand_sheet, "resolving")
        self.assertHasType(self.demand_sheet.resolving, bool)

    def test_root_user(self):
        # dnf.cli.demand.DemandSheet.root_user
        self.assertHasAttr(self.demand_sheet, "root_user")
        self.assertHasType(self.demand_sheet.root_user, bool)

    def test_sack_activation(self):
        # dnf.cli.demand.DemandSheet.sack_activation
        self.assertHasAttr(self.demand_sheet, "sack_activation")
        self.assertHasType(self.demand_sheet.sack_activation, bool)

    def test_load_system_repo(self):
        # dnf.cli.demand.DemandSheet.load_system_repo
        self.assertHasAttr(self.demand_sheet, "load_system_repo")
        self.assertHasType(self.demand_sheet.load_system_repo, bool)

    def test_success_exit_status(self):
        # dnf.cli.demand.DemandSheet.success_exit_status
        self.assertHasAttr(self.demand_sheet, "success_exit_status")
        self.assertHasType(self.demand_sheet.success_exit_status, int)

    def test_cacheonly(self):
        # dnf.cli.demand.DemandSheet.cacheonly
        self.assertHasAttr(self.demand_sheet, "cacheonly")
        self.assertHasType(self.demand_sheet.cacheonly, bool)

    def test_fresh_metadata(self):
        # dnf.cli.demand.DemandSheet.fresh_metadata
        self.assertHasAttr(self.demand_sheet, "fresh_metadata")
        self.assertHasType(self.demand_sheet.fresh_metadata, bool)

    def test_freshest_metadata(self):
        # dnf.cli.demand.DemandSheet.freshest_metadata
        self.assertHasAttr(self.demand_sheet, "freshest_metadata")
        self.assertHasType(self.demand_sheet.freshest_metadata, bool)

    def test_changelogs(self):
        # dnf.cli.demand.DemandSheet.changelogs
        self.assertHasAttr(self.demand_sheet, "changelogs")
        self.assertHasType(self.demand_sheet.changelogs, bool)

    def test_transaction_display(self):
        # dnf.cli.demand.DemandSheet.transaction_display
        self.assertHasAttr(self.demand_sheet, "transaction_display")
        self.assertHasType(self.demand_sheet.changelogs, object)

    def test_plugin_filtering_enabled(self):
        # dnf.cli.demand.DemandSheet.plugin_filtering_enabled
        self.assertHasAttr(self.demand_sheet, "plugin_filtering_enabled")
        self.assertHasType(self.demand_sheet.plugin_filtering_enabled, object)



# -*- coding: utf-8 -*-

# Copyright (C) 2012-2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

from __future__ import absolute_import
from __future__ import unicode_literals

import dnf.exceptions
import dnf.repo
import dnf.sack

import tests.support
from tests.support import mock


class SackTest(tests.support.DnfBaseTestCase):

    REPOS = []

    def test_rpmdb_version(self):
        version = self.sack._rpmdb_version()
        self.assertIsNotNone(version)
        expected = "%s:%s" % (tests.support.TOTAL_RPMDB_COUNT, tests.support.RPMDB_CHECKSUM)
        self.assertEqual(version, expected)

    def test_excludepkgs(self):
        self.base.conf.excludepkgs = ['pepper']
        self.base._setup_excludes_includes()
        peppers = self.base.sack.query().filter(name='pepper').run()
        self.assertLength(peppers, 0)

    def test_exclude(self):
        self.base.conf.exclude = ['pepper']
        self.base._setup_excludes_includes()
        peppers = self.base.sack.query().filter(name='pepper').run()
        self.assertLength(peppers, 0)

    def test_disable_excludes(self):
        self.base.conf.disable_excludes = ['all']
        self.base.conf.excludepkgs = ['pepper']
        self.base._setup_excludes_includes()
        peppers = self.base.sack.query().filter(name='pepper').run()
        self.assertLength(peppers, 1)

    def test_excludepkgs_glob(self):
        # override base with custom repos
        self.base = tests.support.MockBase('main')
        self.base.repos['main'].excludepkgs = ['pepp*']
        self.base._setup_excludes_includes()
        peppers = self.base.sack.query().filter(name='pepper', reponame='main')
        self.assertLength(peppers, 0)

    def test_excludepkgs_includepkgs(self):
        self.base.conf.excludepkgs = ['*.i?86']
        self.base.conf.includepkgs = ['lib*']
        self.base._setup_excludes_includes()
        peppers = self.base.sack.query().run()
        self.assertLength(peppers, 1)
        self.assertEqual(str(peppers[0]), "librita-1-1.x86_64")

    @mock.patch('dnf.sack._build_sack', lambda x: mock.Mock())
    @mock.patch('dnf.goal.Goal', lambda x: mock.Mock())
    def test_fill_sack(self):
        def raiser():
            raise dnf.exceptions.RepoError()

        r = tests.support.MockRepo('bag', self.base.conf)
        r.enable()
        self.base._repos.add(r)
        r.load = mock.Mock(side_effect=raiser)
        r.skip_if_unavailable = False
        self.assertRaises(dnf.exceptions.RepoError,
                          self.base.fill_sack, load_system_repo=False)
        self.assertTrue(r.enabled)
        self.assertTrue(r._check_config_file_age)



# -*- coding: utf-8 -*-

# Copyright (C) 2014-2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

from __future__ import absolute_import

import libdnf.transaction

import dnf.cli.commands.autoremove as autoremove
from dnf.cli.option_parser import OptionParser

import tests.support


class AutoRemoveCommandTest(tests.support.ResultTestCase):

    REPOS = []
    CLI = "mock"

    def test_run(self):
        q = self.base.sack.query()
        pkgs = list(q.filter(name='librita')) + list(q.filter(name='pepper'))

        self._swdb_begin()
        for pkg in pkgs:
            self.base.history.set_reason(pkg, libdnf.transaction.TransactionItemReason_USER)
        self._swdb_end()

        cmd = autoremove.AutoremoveCommand(self.cli)
        parser = OptionParser()
        parser.parse_main_args(['autoremove', '-y'])
        parser.parse_command_args(cmd, ['autoremove', '-y'])
        cmd.run()
        inst, rem = self.installed_removed(self.base)
        self.assertEmpty(inst)
        removed = ('librita-1-1.i686',
                   'librita-1-1.x86_64',
                   'pepper-20-0.x86_64')
        self.assertCountEqual((map(str, pkgs)), removed)



# -*- coding: utf-8 -*-

# Copyright (C) 2013-2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

from __future__ import unicode_literals

import dnf.repodict

import tests.support
from tests.support import TestCase


class TestRepoDict(TestCase):
    def setUp(self):
        conf = tests.support.FakeConf()
        self.x = tests.support.MockRepo('x', conf)
        self.xx = tests.support.MockRepo('xx', conf)
        self.y = tests.support.MockRepo('y', conf)
        self.z = tests.support.MockRepo('z', conf)

        self.repos = dnf.repodict.RepoDict()
        self.repos.add(self.x)
        self.repos.add(self.xx)
        self.repos.add(self.y)
        self.repos.add(self.z)
        self.full_set = {self.x, self.xx, self.y, self.z}

    def test_any_enabled(self):
        self.assertTrue(self.repos._any_enabled())
        self.repos.get_matching("*").disable()
        self.assertFalse(self.repos._any_enabled())

    def test_get_matching(self):
        self.assertEqual(self.repos['x'], self.x)
        self.assertCountEqual(self.repos.get_matching('*'), self.full_set)
        self.assertCountEqual(self.repos.get_matching('y'), {self.y})
        self.assertCountEqual(self.repos.get_matching('x*'), {self.x, self.xx})

        self.assertCountEqual(self.repos.get_matching('nope'), [])

    def test_iter_enabled(self):
        self.assertCountEqual(self.repos.iter_enabled(), self.full_set)
        self.repos.get_matching('x*').disable()
        self.assertCountEqual(self.repos.iter_enabled(), {self.y, self.z})

    def test_all(self):
        self.assertCountEqual(self.repos.all(), self.full_set)



# package.py
# Module defining the dnf.Package class.
#
# Copyright (C) 2012-2016 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

""" Contains the dnf.Package class. """

from __future__ import absolute_import
from __future__ import unicode_literals

from dnf.i18n import _

import binascii
import dnf.rpm
import dnf.yum.misc
import hawkey
import logging
import os

logger = logging.getLogger("dnf")


class Package(hawkey.Package):
    """ Represents a package. #:api """

    DEBUGINFO_SUFFIX = "-debuginfo"  # :api
    DEBUGSOURCE_SUFFIX = "-debugsource"  # :api

    def __init__(self, initobject, base):
        super(Package, self).__init__(initobject)
        self.base = base
        self._priv_chksum = None
        self._repo = None
        self._priv_size = None

    @property
    def _chksum(self):
        if self._priv_chksum:
            return self._priv_chksum
        if self._from_cmdline:
            chksum_type = dnf.yum.misc.get_default_chksum_type()
            chksum_val = dnf.yum.misc.checksum(chksum_type, self.location)
            return (hawkey.chksum_type(chksum_type),
                    binascii.unhexlify(chksum_val))
        return super(Package, self).chksum

    @_chksum.setter
    def _chksum(self, val):
        self._priv_chksum = val

    @property
    def _from_cmdline(self):
        return self.reponame == hawkey.CMDLINE_REPO_NAME

    @property
    def _from_system(self):
        return self.reponame == hawkey.SYSTEM_REPO_NAME

    @property
    def _from_repo(self):
        pkgrepo = None
        if self._from_system:
            pkgrepo = self.base.history.repo(self)
        else:
            pkgrepo = {}
        if pkgrepo:
            return '@' + pkgrepo
        return self.reponame

    @property
    def _header(self):
        return dnf.rpm._header(self.localPkg())

    @property
    def _size(self):
        if self._priv_size:
            return self._priv_size
        return super(Package, self).size

    @_size.setter
    def _size(self, val):
        self._priv_size = val

    @property
    def _pkgid(self):
        if self.hdr_chksum is None:
            return None
        (_, chksum) = self.hdr_chksum
        return binascii.hexlify(chksum)

    @property
    def source_name(self):
        # :api
        """
        returns name of source package
        e.g. krb5-libs -> krb5
        """
        if self.sourcerpm is not None:
            # trim suffix first
            srcname = dnf.util.rtrim(self.sourcerpm, ".src.rpm")
            # sourcerpm should be in form of name-version-release now, so we
            # will strip the two rightmost parts separated by dash.
            # Using rtrim with version and release of self is not sufficient
            # because the package can have different version to the source
            # package.
            srcname = srcname.rsplit('-', 2)[0]
        else:
            srcname = None
        return srcname

    @property
    def debug_name(self):
        # :api
        """
        Returns name of the debuginfo package for this package.
        If this package is a debuginfo package, returns its name.
        If this package is a debugsource package, returns the debuginfo package
        for the base package.
        e.g. kernel-PAE -> kernel-PAE-debuginfo
        """
        if self.name.endswith(self.DEBUGINFO_SUFFIX):
            return self.name

        name = self.name
        if self.name.endswith(self.DEBUGSOURCE_SUFFIX):
            name = name[:-len(self.DEBUGSOURCE_SUFFIX)]

        return name + self.DEBUGINFO_SUFFIX

    @property
    def debugsource_name(self):
        # :api
        """
        Returns name of the debugsource package for this package.
        e.g. krb5-libs -> krb5-debugsource
        """
        # assuming self.source_name is None only for a source package
        src_name = self.source_name if self.source_name is not None else self.name
        return src_name + self.DEBUGSOURCE_SUFFIX

    @property
    def source_debug_name(self):
        # :api
        """
        returns name of debuginfo package for source package of given package
        e.g. krb5-libs -> krb5-debuginfo
        """
        # assuming self.source_name is None only for a source package
        src_name = self.source_name if self.source_name is not None else self.name
        return src_name + self.DEBUGINFO_SUFFIX

    @property # yum compatibility attribute
    def idx(self):
        """ Always type it to int, rpm bindings expect it like that. """
        return int(self.rpmdbid)

    @property # yum compatibility attribute
    def repoid(self):
        return self.reponame

    @property # yum compatibility attribute
    def pkgtup(self):
        return (self.name, self.arch, str(self.e), self.v, self.r)

    @property # yum compatibility attribute
    def repo(self):
        if self._repo:
            return self._repo
        return self.base.repos[self.reponame]

    @repo.setter
    def repo(self, val):
        self._repo = val

    @property
    def reason(self):
        if self.repoid != hawkey.SYSTEM_REPO_NAME:
            return None
        return self.base.history.rpm.get_reason_name(self)

    @property # yum compatibility attribute
    def relativepath(self):
        return self.location

    @property # yum compatibility attribute
    def a(self):
        return self.arch

    @property # yum compatibility attribute
    def e(self):
        return self.epoch

    @property # yum compatibility attribute
    def v(self):
        return self.version

    @property # yum compatibility attribute
    def r(self):
        return self.release

    @property # yum compatibility attribute
    def ui_from_repo(self):
        return self.reponame

    # yum compatibility method
    def evr_eq(self, pkg):
        return self.evr_cmp(pkg) == 0

    # yum compatibility method
    def evr_gt(self, pkg):
        return self.evr_cmp(pkg) > 0

    # yum compatibility method
    def evr_lt(self, pkg):
        return self.evr_cmp(pkg) < 0

    # yum compatibility method
    def getDiscNum(self):
        return self.medianr

    # yum compatibility method
    def localPkg(self):
        """ Package's location in the filesystem.

            For packages in remote repo returns where the package will be/has
            been downloaded.
        """
        if self._from_cmdline:
            return self.location
        loc = self.location
        if self.repo._repo.isLocal() and self.baseurl and self.baseurl.startswith('file://'):
            return os.path.join(self.get_local_baseurl(), loc.lstrip("/"))
        if not self._is_local_pkg():
            loc = os.path.basename(loc)
        return os.path.join(self.pkgdir, loc.lstrip("/"))

    def remote_location(self, schemes=('http', 'ftp', 'file', 'https')):
        # :api
        """
        The location from where the package can be downloaded from

        :param schemes: list of allowed protocols. Default is ('http', 'ftp', 'file', 'https')
        :return: location (string) or None
        """
        return self.repo.remote_location(self.location, schemes)

    def _is_local_pkg(self):
        if self.repoid == "@System":
            return True
        if '://' in self.location and not self.location.startswith('file://'):
            # the package has a remote URL as its location
            return False
        return self._from_cmdline or \
            (self.repo._repo.isLocal() and (not self.baseurl or self.baseurl.startswith('file://')))

    @property
    def pkgdir(self):
        if (self.repo._repo.isLocal() and not self._is_local_pkg()):
            return self.repo.cache_pkgdir()
        else:
            return self.repo.pkgdir

    # yum compatibility method
    def returnIdSum(self):
        """ Return the chksum type and chksum string how the legacy yum expects
            it.
        """
        if self._chksum is None:
            return (None, None)
        (chksum_type, chksum) = self._chksum
        return (hawkey.chksum_name(chksum_type), binascii.hexlify(chksum).decode())

    # yum compatibility method
    def verifyLocalPkg(self):
        if self._from_system:
            raise ValueError("Can not verify an installed package.")
        if self._from_cmdline:
            return True # local package always verifies against itself
        (chksum_type, chksum) = self.returnIdSum()
        real_sum = dnf.yum.misc.checksum(chksum_type, self.localPkg(),
                                         datasize=self._size)
        if real_sum != chksum:
            logger.debug(_('%s: %s check failed: %s vs %s'),
                         self, chksum_type, real_sum, chksum)
            return False
        return True



# -*- coding: utf-8 -*-

# Copyright (C) 2012-2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

from __future__ import absolute_import
from __future__ import unicode_literals

import tests.support


class ProvidesTest(tests.support.DnfBaseTestCase):

    REPOS = ['main']

    def test_file(self):
        self.assertLength(self.base.provides("*ais*smile")[0], 1)
        self.assertLength(self.base.provides("/raised/smile")[0], 1)

    def test_name(self):
        self.assertLength(self.base.provides("henry(the_horse)")[0], 1)
        self.assertLength(self.base.provides("lotus")[0], 2)

    def test_glob(self):
        self.assertLength(self.base.provides("henry(*)")[0], 1)
        self.assertEqual(set(self.base.provides("dup*")[0]), set(self.base.provides('dup')[0]))
        self.assertEqual(set(self.base.provides(["dup*"])[0]), set(self.base.provides('dup')[0]))



# -*- coding: utf-8 -*-

# Copyright (C) 2013-2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import operator

import libcomps
import libdnf.transaction

import dnf.comps
import dnf.exceptions
import dnf.persistor
import dnf.util

import tests.support
from tests.support import mock


TRANSLATION = u"""Tato skupina zahrnuje nejmenší možnou množinu balíčků. Je vhodná například na instalace malých routerů nebo firewallů."""


class LangsTest(tests.support.TestCase):
    @mock.patch('locale.getlocale', return_value=('cs_CZ', 'UTF-8'))
    def test_get(self, _unused):
        langs = dnf.comps._Langs().get()
        self.assertEqual(langs, ['cs_CZ.UTF-8', 'cs_CZ', 'cs.UTF-8', 'cs', 'C'])


class CompsTest(tests.support.TestCase):
    def setUp(self):
        comps = dnf.comps.Comps()
        comps._add_from_xml_filename(tests.support.COMPS_PATH)
        self.comps = comps

    def test_by_pattern(self):
        comps = self.comps
        self.assertLength(comps.groups_by_pattern('Base'), 1)
        self.assertLength(comps.groups_by_pattern('*'), tests.support.TOTAL_GROUPS)
        self.assertLength(comps.groups_by_pattern('Solid*'), 1)

        group = dnf.util.first(comps.groups_by_pattern('Base'))
        self.assertIsInstance(group, dnf.comps.Group)

    def test_categories(self):
        cat = self.comps.categories[0]
        self.assertEqual(cat.name_by_lang['cs'], u'Základ systému')
        self.assertEqual(cat.desc_by_lang['de'],
                         u'Verschiedene Kernstücke des Systems.')
        self.assertCountEqual((id_.name for id_ in cat.group_ids),
                              ('base', ))
        self.assertCountEqual((id_.default for id_ in cat.group_ids),
                              (False, ))
        self.assertTrue(all(isinstance(grp, dnf.comps.Group)
                            for grp in cat.groups_iter()))

    def test_environments(self):
        env = self.comps.environments[0]
        self.assertEqual(env.name_by_lang['cs'], u'Prostředí Sugar')
        self.assertEqual(env.desc_by_lang['de'],

                         u'Eine Software-Spielwiese zum Lernen des Lernens.')
        self.assertCountEqual((id_.name for id_ in env.group_ids),
                              ('somerset', 'Peppers'))
        self.assertEqual(2, len(env.mandatory_groups))
        self.assertTrue(all(isinstance(grp, dnf.comps.Group)
                            for grp in env.mandatory_groups))
        self.assertCountEqual((id_.default for id_ in env.group_ids),
                              (True, False))
        self.assertCountEqual((id_.name for id_ in env.option_ids),
                              ('base',))
        self.assertEqual(1, len(env.optional_groups))
        self.assertTrue(all(isinstance(grp, dnf.comps.Group)
                            for grp in env.optional_groups))
        self.assertTrue(all(isinstance(grp, dnf.comps.Group)
                            for grp in env.groups_iter()))

    def test_groups(self):
        g = self.comps.group_by_pattern('base')
        self.assertTrue(g.visible)
        g = self.comps.group_by_pattern('somerset')
        self.assertFalse(g.visible)

    def test_group_packages(self):
        g = self.comps.group_by_pattern('base')
        self.assertCountEqual(map(operator.attrgetter('name'), g.packages_iter()),
                              ('tour', 'pepper'))

    def test_iteration(self):
        comps = self.comps
        self.assertEqual([g.name for g in comps.groups_iter()],
                         ['Base', 'Solid Ground', "Pepper's", "Broken Group", None])
        self.assertEqual([c.name for c in comps.categories_iter()],
                         ['Base System'])
        g = dnf.util.first(comps.groups_iter())
        self.assertEqual(g.desc_by_lang['cs'], TRANSLATION)

    def test_group_display_order(self):
        self.assertEqual([g.name for g in self.comps.groups],
                         ["Pepper's", 'Base', 'Solid Ground', 'Broken Group', None])

    def test_packages(self):
        comps = self.comps
        group = dnf.util.first(comps.groups_iter())
        self.assertSequenceEqual([pkg.name for pkg in group.packages],
                                 (u'pepper', u'tour'))
        self.assertSequenceEqual([pkg.name for pkg in group.mandatory_packages],
                                 (u'pepper', u'tour'))

    def test_size(self):
        comps = self.comps
        self.assertLength(comps, 7)
        self.assertLength(comps.groups, tests.support.TOTAL_GROUPS)
        self.assertLength(comps.categories, 1)
        self.assertLength(comps.environments, 1)

    @mock.patch('locale.getlocale', return_value=('cs_CZ', 'UTF-8'))
    def test_ui_name(self, _unused):
        comps = self.comps
        group = dnf.util.first(comps.groups_by_pattern('base'))
        self.assertEqual(group.ui_name, u'Kritická cesta (Základ)')

    @mock.patch('locale.getlocale', return_value=('cs_CZ', 'UTF-8'))
    def test_ui_desc(self, _unused):
        comps = self.comps
        env = dnf.util.first(comps.environments_by_pattern('sugar-*'))
        self.assertEqual(env.ui_description, u'Software pro výuku o vyučování.')


class PackageTest(tests.support.TestCase):
    def test_instance(self):
        lc_pkg = libcomps.Package('weather', libcomps.PACKAGE_TYPE_OPTIONAL)
        pkg = dnf.comps.Package(lc_pkg)
        self.assertEqual(pkg.name, 'weather')
        self.assertEqual(pkg.option_type, dnf.comps.OPTIONAL)


class TestTransactionBunch(tests.support.TestCase):

    def test_adding(self):
        t1 = dnf.comps.TransactionBunch()
        t1.install = {'right'}
        t1.upgrade = {'tour'}
        t1.remove = {'pepper'}
        t2 = dnf.comps.TransactionBunch()
        t2.install = {'pepper'}
        t2.upgrade = {'right'}
        t1 += t2
        self.assertTransEqual(t1.install, ('right', 'pepper'))
        self.assertTransEqual(t1.upgrade, ('tour', 'right'))
        self.assertEmpty(t1.remove)


class SolverGroupTest(tests.support.DnfBaseTestCase):

    REPOS = []
    COMPS = True
    COMPS_SOLVER = True

    def test_install(self):
        group_id = 'base'
        trans = self.solver._group_install(group_id, dnf.comps.MANDATORY)
        self.assertLength(trans.install, 2)
        self._swdb_commit()

        swdb_group = self.history.group.get(group_id)
        self.assertCountEqual([i.getName() for i in swdb_group.getPackages()], ['pepper', 'tour'])
        self.assertEqual(swdb_group.getPackageTypes(), dnf.comps.MANDATORY)

    def test_removable_pkg(self):
        comps_group = self.comps.group_by_pattern('base')
        self.solver._group_install(comps_group.id, dnf.comps.MANDATORY, [])

        tsis = []

        pkg1 = self.base.sack.query().filter(name="pepper", epoch=0, version="20", release="0", arch="x86_64")[0]
        self.history.rpm.add_install(pkg1, reason=libdnf.transaction.TransactionItemReason_GROUP)

        pkg3 = self.base.sack.query().filter(name="tour", version="5", release="0", arch="noarch")[0]
        self.history.rpm.add_install(pkg3, reason=libdnf.transaction.TransactionItemReason_GROUP)

        group_id = "dupl"
        swdb_group = self.history.group.new(group_id, group_id, group_id, dnf.comps.DEFAULT)
        swdb_group.addPackage("tour", True, dnf.comps.MANDATORY)
        self.history.group.install(swdb_group)
        self._swdb_commit(tsis)

        # pepper is in single group with reason "group"
        self.assertTrue(self.solver._removable_pkg('pepper'))
        # right's reason is "dep"
        self.assertFalse(self.solver._removable_pkg('right'))
        # tour appears in more than one group
        self.assertFalse(self.solver._removable_pkg('tour'))

        swdb_group = self.history.group.get(group_id)
        self.history.group.remove(swdb_group)

        # tour appears only in one group now
        self.assertTrue(self.solver._removable_pkg('tour'))

    def test_remove(self):
        grp = self.comps.group_by_pattern('base')
        self.solver._group_install(grp.id, dnf.comps.MANDATORY, [])

        grps = self.history.group.search_by_pattern('base')
        for grp in grps:
            self.solver._group_remove(grp)

        # need to load groups again - loaded object is stays the same
        grps = self.history.group.search_by_pattern('base')
        for grp in grps:
            self.assertFalse(grp.installed)

    def test_upgrade(self):
        # setup of the "current state"
        group_id = 'base'

        swdb_group = self.history.group.new(group_id, group_id, group_id, dnf.comps.MANDATORY)
        for pkg_name in ['pepper', 'handerson']:
            swdb_group.addPackage(pkg_name, True, dnf.comps.MANDATORY)
        self.history.group.install(swdb_group)
        self._swdb_commit()

        swdb_group = self.history.group.get(group_id)
        self.assertCountEqual([i.getName() for i in swdb_group.getPackages()], ('handerson', 'pepper'))

        comps_group = self.comps.group_by_pattern(group_id)

        trans = self.solver._group_upgrade(group_id)
        self.assertTransEqual(trans.install, ('tour',))
        self.assertTransEqual(trans.remove, ('handerson',))
        self.assertTransEqual(trans.upgrade, ('pepper',))
        self._swdb_commit()

        swdb_group = self.history.group.get(group_id)
        self.assertCountEqual([i.getName() for i in swdb_group.getPackages()], ('tour', 'pepper'))

class SolverEnvironmentTest(tests.support.DnfBaseTestCase):

    REPOS = []
    COMPS = True
    COMPS_SOLVER = True

    def test_install(self):
        env_id = 'sugar-desktop-environment'
        env = self.comps._environment_by_id(env_id)
        trans = self.solver._environment_install(env_id, dnf.comps.MANDATORY, [])
        self._swdb_commit()

        self.assertCountEqual([pkg.name for pkg in trans.install], ('pepper', 'trampoline', 'hole', 'lotus'))

        sugar = self.history.env.get(env_id)
        self.assertCountEqual([i.getGroupId() for i in sugar.getGroups()], ('Peppers', 'somerset', 'base'))

        somerset = self.history.group.get('somerset')
        self.assertIsNotNone(somerset)
        self.assertEqual(somerset.getPackageTypes(), dnf.comps.MANDATORY)

        base = self.history.group.get('base')
        self.assertEqual(base, None)

    def test_remove(self):
        env_id = 'sugar-desktop-environment'
        comps_env = self.comps.environment_by_pattern(env_id)

        self.solver._environment_install(comps_env.id, dnf.comps.MANDATORY, [])
        self._swdb_commit()

        swdb_env = self.history.env.get(comps_env.id)
        self.assertEqual(swdb_env.getPackageTypes(), dnf.comps.MANDATORY)
        group_ids = [i.getGroupId() for i in swdb_env.getGroups()]

        self.solver._environment_remove(comps_env.id)
        self._swdb_commit()

        swdb_env = self.history.env.get(comps_env.id)
        # if swdb_env is None, then it's removed
        self.assertIsNone(swdb_env)
        # test if also all groups were removed
        for group_id in group_ids:
            swdb_group = self.history.group.get(group_id)
            self.assertIsNone(swdb_group)

        # install it again with different pkg_types
        self.solver._environment_install(comps_env.id, dnf.comps.OPTIONAL, [])
        self._swdb_commit()

        swdb_env = self.history.env.get(comps_env.id)
        self.assertIsNotNone(swdb_env)
        self.assertEqual(swdb_env.getPackageTypes(), dnf.comps.OPTIONAL)

        group_ids = [i.getGroupId() for i in swdb_env.getGroups()]
        self.assertTrue(len(group_ids))
        for group_id in group_ids:
            swdb_group = self.history.group.get(group_id)
            if group_id == "base" and swdb_group is None:
                continue
            self.assertEqual(swdb_group.getPackageTypes(), dnf.comps.OPTIONAL)

    def test_upgrade(self):
        """Upgrade environment, the one group it knows is no longer installed."""
        env_id = "sugar-desktop-environment"
        comps_env = self.comps.environment_by_pattern(env_id)

        self.solver._environment_install(comps_env.id, dnf.comps.ALL_TYPES, [])
        self._swdb_commit()

        swdb_env = self.history.env.get(comps_env.id)
        self.assertNotEqual(swdb_env, None)

        # create a new transaction item for group Peppers with no packages
        self._swdb_commit()
        swdb_group = self.history.group.get('Peppers')
        swdb_group = self.history.group.new(swdb_group.getGroupId(), swdb_group.getName(), swdb_group.getTranslatedName(), swdb_group.getPackageTypes())
        self.history.group.install(swdb_group)
        self._swdb_commit()

        trans = self.solver._environment_upgrade(comps_env.id)
        self._swdb_commit()
        self.assertTransEqual(trans.install, ('hole', 'lotus'))
        self.assertTransEqual(trans.upgrade, ('pepper', 'trampoline', 'lotus'))
        self.assertEmpty(trans.remove)



# -*- coding: utf-8 -*-

# Copyright (C) 2012-2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

from __future__ import absolute_import
from __future__ import unicode_literals

import itertools

import libdnf.transaction

import tests.support


class List(tests.support.DnfBaseTestCase):

    REPOS = ["main", "updates"]

    def test_doPackageLists_reponame(self):
        """Test whether packages are filtered by the reponame."""
        reponame = 'main'
        lists = self.base._do_package_lists(reponame=reponame)

        pkgs = itertools.chain.from_iterable(lists.all_lists().values())
        self.assertCountEqual({pkg.reponame for pkg in pkgs}, {reponame})

        assert len(set(pkg.reponame for pkg in self.base.sack.query())) > 1, \
            ('the base must contain packages from multiple repos, '
             'otherwise the test makes no sense')

    def test_list_installed(self):
        ypl = self.base._do_package_lists('installed')
        self.assertEqual(len(ypl.installed), tests.support.TOTAL_RPMDB_COUNT)

    def test_list_installed_reponame(self):
        """Test whether only packages installed from the repository are listed."""
        expected = self.base.sack.query().installed().filter(name={'pepper', 'librita'})
        tsis = []
        for pkg in expected:
            pkg._force_swdb_repoid = "main"
            self.history.rpm.add_install(pkg)
        self._swdb_commit(tsis)

        lists = self.base._do_package_lists('installed', reponame='main')

        self.assertCountEqual(lists.installed, expected)

    def test_list_updates(self):
        ypl = self.base._do_package_lists('upgrades')
        self.assertEqual(len(ypl.updates), tests.support.UPDATES_NSOLVABLES - 2)
        pkg = ypl.updates[0]
        self.assertEqual(pkg.name, "hole")
        ypl = self.base._do_package_lists('upgrades', ["pepper"])
        self.assertEqual(len(ypl.updates), 1)
        ypl = self.base._do_package_lists('upgrades', ["mrkite"])
        self.assertEqual(len(ypl.updates), 0)

        ypl = self.base._do_package_lists('upgrades', ["hole"])
        self.assertEqual(len(ypl.updates), 1)

    def test_lists_multiple(self):
        ypl = self.base._do_package_lists('upgrades', ['pepper', 'hole'])
        self.assertLength(ypl.updates, 2)


class TestListAllRepos(tests.support.DnfBaseTestCase):

    REPOS = ["main", "updates"]

    def setUp(self):
        super(TestListAllRepos, self).setUp()
        self.base.conf.multilib_policy = "all"

    def test_list_pattern(self):
        ypl = self.base._do_package_lists('all', ['hole'])
        self.assertLength(ypl.installed, 1)
        self.assertLength(ypl.available, 2)

    def test_list_pattern_arch(self):
        ypl = self.base._do_package_lists('all', ['hole.x86_64'])
        self.assertLength(ypl.installed, 1)
        self.assertLength(ypl.available, 1)

    def test_list_available(self):
        ypl = self.base._do_package_lists('available', ['hole'], showdups=False)
        self.assertCountEqual(map(str, ypl.available), ('hole-2-1.i686',
                                                        'hole-2-1.x86_64'))

        ypl = self.base._do_package_lists('available', ['hole'], showdups=True)
        self.assertCountEqual(map(str, ypl.available), ('hole-2-1.i686',
                                                        'hole-2-1.x86_64',
                                                        'hole-1-2.x86_64'))



# -*- coding: utf-8 -*-


from __future__ import absolute_import
from __future__ import unicode_literals

import dnf.exceptions

from .common import TestCase


class DnfExceptionsApiTest(TestCase):

    def test_deprectation_warning(self):
        # dnf.exceptions.DeprecationWarning
        self.assertHasAttr(dnf.exceptions, "DeprecationWarning")
        self.assertHasType(dnf.exceptions.DeprecationWarning(), DeprecationWarning)

    def test_error(self):
        # dnf.exceptions.Error
        self.assertHasAttr(dnf.exceptions, "Error")
        ex = dnf.exceptions.Error(value=None)
        self.assertHasType(ex, Exception)

    def test_comps_error(self):
        # dnf.exceptions.CompsError
        self.assertHasAttr(dnf.exceptions, "CompsError")
        ex = dnf.exceptions.CompsError(value=None)
        self.assertHasType(ex, dnf.exceptions.Error)

    def test_depsolve_error(self):
        # dnf.exceptions.DepsolveError
        self.assertHasAttr(dnf.exceptions, "DepsolveError")
        ex = dnf.exceptions.DepsolveError(value=None)
        self.assertHasType(ex, dnf.exceptions.Error)

    def test_download_error(self):
        # dnf.exceptions.DownloadError
        self.assertHasAttr(dnf.exceptions, "DownloadError")
        ex = dnf.exceptions.DownloadError(errmap=None)
        self.assertHasType(ex, dnf.exceptions.Error)

    def test_marking_error(self):
        # dnf.exceptions.MarkinError
        self.assertHasAttr(dnf.exceptions, "MarkingError")
        ex = dnf.exceptions.MarkingError(value=None, pkg_spec=None)
        self.assertHasType(ex, dnf.exceptions.Error)

    def test_marking_errors(self):
        # dnf.exceptions.MarkinErrors
        self.assertHasAttr(dnf.exceptions, "MarkingErrors")
        ex = dnf.exceptions.MarkingErrors(
            no_match_group_specs=(),
            error_group_specs=(),
            no_match_pkg_specs=(),
            error_pkg_specs=(),
            module_depsolv_errors=()
        )
        self.assertHasType(ex, dnf.exceptions.Error)

    def test_repo_error(self):
        # dnf.exceptions.RepoError
        self.assertHasAttr(dnf.exceptions, "RepoError")
        ex = dnf.exceptions.RepoError()
        self.assertHasType(ex, dnf.exceptions.Error)



# -*- coding: utf-8 -*-

# Copyright (C) 2012-2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

from __future__ import absolute_import
from __future__ import unicode_literals

import argparse
import os
import re
from argparse import Namespace

import dnf.cli.cli
import dnf.conf
import dnf.goal
import dnf.repo
import dnf.repodict

import tests.support
from tests.support import mock


VERSIONS_OUTPUT = """\
  Installed: pepper-0:20-0.x86_64 at Thu Jan  1 00:00:00 1970
  Built    :  at Thu Jan  1 00:00:00 1970

  Installed: tour-0:5-0.noarch at Thu Jan  1 00:00:00 1970
  Built    :  at Thu Jan  1 00:00:00 1970
"""


class VersionStringTest(tests.support.DnfBaseTestCase):

    REPOS = []

    def test_print_versions(self):
        output = tests.support.MockOutput()
        with mock.patch('sys.stdout') as stdout,\
                mock.patch('dnf.sack._rpmdb_sack', return_value=self.base.sack):
            dnf.cli.cli.print_versions(['pepper', 'tour'], self.base, output)
        written = ''.join([mc[1][0] for mc in stdout.method_calls
                           if mc[0] == 'write'])
        self.assertEqual(written, VERSIONS_OUTPUT)


@mock.patch('dnf.cli.cli.logger', new_callable=tests.support.mock_logger)
class BaseCliTest(tests.support.ResultTestCase):

    REPOS = ["main", "updates"]
    BASE_CLI = True
    INIT_SACK = True

    def setUp(self):
        super(BaseCliTest, self).setUp()
        self.base.output.term = tests.support.MockTerminal()
        self.base.downgrade_to = mock.Mock(wraps=self.base.downgrade_to)

    def test_downgradePkgs(self, logger):
        self.base.downgradePkgs(('tour',))

        self.assertEqual(self.base.downgrade_to.mock_calls, [mock.call('tour', strict=False)])
        self.assertEqual(logger.mock_calls, [])

    def test_downgradePkgs_notfound(self, logger):
        with self.assertRaises(dnf.exceptions.Error) as ctx:
            self.base.downgradePkgs(('non-existent',))
        self.assertEqual(str(ctx.exception), 'No packages marked for downgrade.')

        self.assertEqual(self.base.downgrade_to.mock_calls,
                         [mock.call('non-existent', strict=False)])
        self.assertEqual(logger.mock_calls,
                         [mock.call.info('No package %s available.',
                                         'non-existent')])

    @mock.patch('dnf.cli.cli._', dnf.pycomp.NullTranslations().ugettext)
    def test_downgradePkgs_notinstalled(self, logger):
        tests.support.ObjectMatcher(dnf.package.Package, {'name': 'lotus'})

        with self.assertRaises(dnf.exceptions.Error) as ctx:
            self.base.downgradePkgs(('lotus',))
        self.assertEqual(str(ctx.exception), 'No packages marked for downgrade.')

        self.assertEqual(self.base.downgrade_to.mock_calls, [mock.call('lotus', strict=False)])


@mock.patch('dnf.cli.cli.Cli._read_conf_file')
class CliTest(tests.support.DnfBaseTestCase):

    REPOS = ["main"]
    CLI = "init"

    def setUp(self):
        super(CliTest, self).setUp()
        self.base.output = tests.support.MockOutput()

    def test_knows_upgrade(self, _):
        upgrade = self.cli.cli_commands['upgrade']
        update = self.cli.cli_commands['update']
        self.assertIs(upgrade, update)

    def test_simple(self, _):
        self.assertFalse(self.base.conf.assumeyes)
        self.cli.configure(['update', '-y'])
        self.assertTrue(self.base.conf.assumeyes)

    def test_glob_options_cmds(self, _):
        params = [
            ['install', '-y', 'pkg1', 'pkg2'],
            ['install', 'pkg1', '-y', 'pkg2'],
            ['install', 'pkg1', 'pkg2', '-y'],
            ['-y', 'install', 'pkg1', 'pkg2']
        ]
        for param in params:
            self.cli.configure(args=param)
            self.assertTrue(self.base.conf.assumeyes)
            self.assertEqual(self.cli.command.opts.command, "install")
            self.assertEqual(self.cli.command.opts.pkg_specs, ["pkg1", "pkg2"])

    def test_configure_repos(self, _):
        opts = Namespace()
        opts.repo = []
        opts.repos_ed = [('*', 'disable'), ('comb', 'enable')]
        opts.cacheonly = True
        opts.nogpgcheck = True
        opts.repofrompath = {}
        self.base._repos = dnf.repodict.RepoDict()
        self.base._repos.add(tests.support.MockRepo('one', self.base.conf))
        self.base._repos.add(tests.support.MockRepo('two', self.base.conf))
        self.base._repos.add(tests.support.MockRepo('comb', self.base.conf))
        self.cli._configure_repos(opts)
        self.assertFalse(self.base.repos['one'].enabled)
        self.assertFalse(self.base.repos['two'].enabled)
        self.assertTrue(self.base.repos['comb'].enabled)
        self.assertFalse(self.base.repos["comb"].gpgcheck)
        self.assertFalse(self.base.repos["comb"].repo_gpgcheck)

    def test_configure_repos_expired(self, _):
        """Ensure that --cacheonly beats the expired status."""
        opts = Namespace()
        opts.repo = []
        opts.repos_ed = []
        opts.cacheonly = True
        opts.repofrompath = {}

        pers = self.base._repo_persistor
        pers.get_expired_repos = mock.Mock(return_value=('one',))
        self.base._repos = dnf.repodict.RepoDict()
        self.base._repos.add(tests.support.MockRepo('one', self.base.conf))
        self.cli._configure_repos(opts)
        # _process_demands() should respect --cacheonly in spite of modified demands
        self.cli.demands.fresh_metadata = False
        self.cli.demands.cacheonly = True
        self.cli._process_demands()
        self.assertEqual(self.base.repos['one']._repo.getSyncStrategy(),
                         dnf.repo.SYNC_ONLY_CACHE)


@mock.patch('dnf.logging.Logging._setup', new=mock.MagicMock)
class ConfigureTest(tests.support.DnfBaseTestCase):

    REPOS = ["main"]
#    CLI = "init"

    def setUp(self):
        super(ConfigureTest, self).setUp()
        self.base._conf = dnf.conf.Conf()
        self.base.output = tests.support.MockOutput()
        self.base._plugins = mock.Mock()
        self.cli = dnf.cli.cli.Cli(self.base)
        self.cli.command = mock.Mock()
        self.conffile = os.path.join(tests.support.dnf_toplevel(), "etc/dnf/dnf.conf")

    @mock.patch('dnf.util.am_i_root', lambda: False)
    def test_configure_user(self):
        """ Test Cli.configure as user."""
        # call setUp() once again *after* am_i_root() is mocked so the cachedir is set as expected
        self.setUp()
        self.base._conf.installroot = self._installroot
        with mock.patch('dnf.rpm.detect_releasever', return_value=69):
            self.cli.configure(['update', '-c', self.conffile])
        reg = re.compile('^' + self._installroot + '/var/tmp/dnf-[.a-zA-Z0-9_-]+$')
        self.assertIsNotNone(reg.match(self.base.conf.cachedir))
        parser = argparse.ArgumentParser()
        expected = "%s update -c %s " % (parser.prog, self.conffile)
        self.assertEqual(self.cli.cmdstring, expected)

    @mock.patch('dnf.util.am_i_root', lambda: True)
    def test_configure_root(self):
        """ Test Cli.configure as root."""
        self.base._conf = dnf.conf.Conf()
        with mock.patch('dnf.rpm.detect_releasever', return_value=69):
            self.cli.configure(['update', '--nogpgcheck', '-c', self.conffile])
        reg = re.compile('^/var/cache/dnf$')
        self.assertIsNotNone(reg.match(self.base.conf.system_cachedir))
        parser = argparse.ArgumentParser()
        expected = "%s update --nogpgcheck -c %s " % (parser.prog, self.conffile)
        self.assertEqual(self.cli.cmdstring, expected)

    def test_configure_verbose(self):
        self.base._conf.installroot = self._installroot
        with mock.patch('dnf.rpm.detect_releasever', return_value=69):
            self.cli.configure(['-v', 'update', '-c', self.conffile])
        parser = argparse.ArgumentParser()
        expected = "%s -v update -c %s " % (parser.prog, self.conffile)
        self.assertEqual(self.cli.cmdstring, expected)
        self.assertEqual(self.base.conf.debuglevel, 6)
        self.assertEqual(self.base.conf.errorlevel, 6)

    @mock.patch('dnf.cli.cli.Cli._parse_commands', new=mock.MagicMock)
    @mock.patch('os.path.exists', return_value=True)
    def test_conf_exists_in_installroot(self, ospathexists):
        with mock.patch('logging.Logger.warning'), \
                mock.patch('dnf.rpm.detect_releasever', return_value=69):
            self.cli.configure(['--installroot', '/roots/dnf', 'update'])
        self.assertEqual(self.base.conf.config_file_path, '/roots/dnf/etc/dnf/dnf.conf')
        self.assertEqual(self.base.conf.installroot, '/roots/dnf')

    @mock.patch('dnf.cli.cli.Cli._parse_commands', new=mock.MagicMock)
    @mock.patch('os.path.exists', return_value=False)
    def test_conf_notexists_in_installroot(self, ospathexists):
        with mock.patch('dnf.rpm.detect_releasever', return_value=69):
            self.cli.configure(['--installroot', '/roots/dnf', 'update'])
        self.assertEqual(self.base.conf.config_file_path, '/etc/dnf/dnf.conf')
        self.assertEqual(self.base.conf.installroot, '/roots/dnf')

    @mock.patch('dnf.cli.cli.Cli._parse_commands', new=mock.MagicMock)
    def test_installroot_with_etc(self):
        """Test that conffile is detected in a new installroot."""
        self.base.extcmds = []

        tlv = tests.support.dnf_toplevel()
        self.cli.configure(['--installroot', tlv, 'update'])
        self.assertEqual(self.base.conf.config_file_path, '%s/etc/dnf/dnf.conf' % tlv)

    def test_installroot_configurable(self):
        """Test that conffile is detected in a new installroot."""

        conf = os.path.join(tests.support.dnf_toplevel(), "tests/etc/installroot.conf")
        self.cli.configure(['-c', conf, '--nogpgcheck', '--releasever', '17', 'update'])
        self.assertEqual(self.base.conf.installroot, '/roots/dnf')



# -*- coding: utf-8 -*-


from __future__ import absolute_import
from __future__ import unicode_literals

import dnf

from .common import TestCase


class DnfConstApiTest(TestCase):
    def test_filename(self):
        # dnf.const.CONF_FILENAME
        self.assertHasAttr(dnf.const, "CONF_FILENAME")
        self.assertHasType(dnf.const.CONF_FILENAME, str)

    def test_group_package_types(self):
        # dnf.const.GROUP_PACKAGE_TYPES
        self.assertHasAttr(dnf.const, "GROUP_PACKAGE_TYPES")
        self.assertHasType(dnf.const.GROUP_PACKAGE_TYPES, tuple)

    def test_persistdir(self):
        # dnf.const.PERSISTDIR
        self.assertHasAttr(dnf.const, "PERSISTDIR")
        self.assertHasType(dnf.const.PERSISTDIR, str)

    def test_pluginconfpath(self):
        # dnf.const.PLUGINCONFPATH
        self.assertHasAttr(dnf.const, "PLUGINCONFPATH")
        self.assertHasType(dnf.const.PLUGINCONFPATH, str)



# -*- coding: utf-8 -*-

# Copyright (C) 2013-2018  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

from __future__ import absolute_import
from __future__ import unicode_literals

import logging
import collections
import gzip
import operator
import os
import tempfile

import dnf.const
import dnf.logging

import tests.support
from tests.support import mock


LogfileEntry = collections.namedtuple('LogfileEntry', ('date', 'time', 'message'))


def _split_logfile_entry(entry):
    record = entry.split(' ')
    datetime = record[0].split('T')
    # strip the trailing '\n' from the message:
    message = ' '.join(record[2:])[:-1]
    return LogfileEntry(date=datetime[0],
                        time=datetime[1], message=message)


def drop_all_handlers():
    for logger_name in ('dnf', 'dnf.rpm'):
        logger = logging.getLogger(logger_name)
        for handler in logger.handlers[:]:
            logger.removeHandler(handler)


class TestLogging(tests.support.TestCase):
    """Tests the logging mechanisms in DNF.

    If it causes a problem in the future that loggers are singletons that don't
    get torn down between tests, look at logging.Manager internals.

    """

    def setUp(self):
        self.logdir = tempfile.mkdtemp(prefix="dnf-logtest-")
        self.log_size = 1024 * 1024
        self.log_rotate = 4
        self.log_compress = False
        self.logging = dnf.logging.Logging()

    def tearDown(self):
        drop_all_handlers()
        dnf.util.rm_rf(self.logdir)

    @staticmethod
    def _bench(logger):
        logger.debug(u"d")
        logger.info(u"i")
        logger.warning(u"w")
        logger.error(u"e")

    def test_level_conversion(self):
        self.assertRaises(AssertionError, dnf.logging._cfg_verbose_val2level, 11)
        self.assertEqual(dnf.logging._cfg_verbose_val2level(0),
                         dnf.logging.SUPERCRITICAL)
        self.assertEqual(dnf.logging._cfg_verbose_val2level(7),
                         dnf.logging.DDEBUG)

    def test_setup(self):
        logger = logging.getLogger("dnf")
        with tests.support.patch_std_streams() as (stdout, stderr):
            self.logging._setup(
                logging.INFO, logging.ERROR, dnf.logging.TRACE,
                self.logdir, self.log_size, self.log_rotate, self.log_compress)
            self._bench(logger)
        self.assertEqual("i\n", stdout.getvalue())
        self.assertEqual("e\n", stderr.getvalue())

    def test_setup_verbose(self):
        logger = logging.getLogger("dnf")
        with tests.support.patch_std_streams() as (stdout, stderr):
            self.logging._setup(
                logging.DEBUG, logging.WARNING, dnf.logging.TRACE,
                self.logdir, self.log_size, self.log_rotate, self.log_compress)
            self._bench(logger)
        self.assertEqual("d\ni\n", stdout.getvalue())
        self.assertEqual("w\ne\n", stderr.getvalue())

    @mock.patch('dnf.logging.Logging._setup')
    def test_setup_from_dnf_conf(self, setup_m):
        conf = mock.Mock(
            debuglevel=2, errorlevel=3, logfilelevel=2, logdir=self.logdir,
            log_size=self.log_size, log_rotate=self.log_rotate, log_compress=self.log_compress)
        self.logging._setup_from_dnf_conf(conf)
        self.assertEqual(setup_m.call_args, mock.call(dnf.logging.INFO,
                                                      dnf.logging.WARNING,
                                                      dnf.logging.INFO,
                                                      self.logdir,
                                                      self.log_size,
                                                      self.log_rotate,
                                                      self.log_compress))
        conf = mock.Mock(
            debuglevel=6, errorlevel=6, logfilelevel=6, logdir=self.logdir,
            log_size=self.log_size, log_rotate=self.log_rotate, log_compress=self.log_compress)
        self.logging._setup_from_dnf_conf(conf)
        self.assertEqual(setup_m.call_args, mock.call(dnf.logging.DEBUG,
                                                      dnf.logging.WARNING,
                                                      dnf.logging.DEBUG,
                                                      self.logdir,
                                                      self.log_size,
                                                      self.log_rotate,
                                                      self.log_compress))

    def test_file_logging(self):
        # log nothing to the console:
        self.logging._setup(
            dnf.logging.SUPERCRITICAL, dnf.logging.SUPERCRITICAL, dnf.logging.TRACE,
            self.logdir, self.log_size, self.log_rotate, self.log_compress)
        logger = logging.getLogger("dnf")
        with tests.support.patch_std_streams() as (stdout, stderr):
            logger.info("i")
            logger.critical("c")
        self.assertEqual(stdout.getvalue(), '')
        self.assertEqual(stderr.getvalue(), '')
        # yet the file should contain both the entries:
        logfile = os.path.join(self.logdir, "dnf.log")
        self.assertFile(logfile)
        with open(logfile) as f:
            msgs = map(operator.attrgetter("message"),
                       map(_split_logfile_entry, f.readlines()))
        self.assertSequenceEqual(list(msgs), [dnf.const.LOG_MARKER, 'i', 'c'])

    def test_rpm_logging(self):
        # log everything to the console:
        self.logging._setup(
            dnf.logging.SUBDEBUG, dnf.logging.SUBDEBUG, dnf.logging.TRACE,
            self.logdir, self.log_size, self.log_rotate, self.log_compress)
        logger = logging.getLogger("dnf.rpm")
        with tests.support.patch_std_streams() as (stdout, stderr):
            logger.info('rpm transaction happens.')
        # rpm logger never outputs to the console:
        self.assertEqual(stdout.getvalue(), "")
        self.assertEqual(stderr.getvalue(), "")
        logfile = os.path.join(self.logdir, "dnf.rpm.log")
        self.assertFile(logfile)
        with open(logfile) as f:
            msgs = map(operator.attrgetter("message"),
                       map(_split_logfile_entry, f.readlines()))
        self.assertSequenceEqual(
            list(msgs),
            [dnf.const.LOG_MARKER, 'rpm transaction happens.']
        )

    def test_setup_only_once(self):
        logger = logging.getLogger("dnf")
        self.assertLength(logger.handlers, 0)
        self.logging._setup(
            dnf.logging.SUBDEBUG, dnf.logging.SUBDEBUG, dnf.logging.TRACE,
            self.logdir, self.log_size, self.log_rotate, self.log_compress)
        cnt = len(logger.handlers)
        self.assertGreater(cnt, 0)
        self.logging._setup(
            dnf.logging.SUBDEBUG, dnf.logging.SUBDEBUG,
            self.logdir, self.log_size, self.log_rotate, self.log_compress)
        # no new handlers
        self.assertEqual(cnt, len(logger.handlers))

    def test_log_compression(self):
        # log nothing to the console and set log_compress=True and log_size to minimal size, so it's always rotated:
        self.logging._setup(
            dnf.logging.SUPERCRITICAL, dnf.logging.SUPERCRITICAL, dnf.logging.TRACE,
            self.logdir, log_size=1, log_rotate=self.log_rotate, log_compress=True)
        logger = logging.getLogger("dnf")
        with tests.support.patch_std_streams() as (stdout, stderr):
            logger.info("i")
            logger.critical("c")
        logfile = os.path.join(self.logdir, "dnf.log")
        self.assertFile(logfile)
        with open(logfile) as f:
            msgs = map(operator.attrgetter("message"),
                       map(_split_logfile_entry, f.readlines()))
        self.assertSequenceEqual(list(msgs), ['c'])
        logfile_rotated = os.path.join(self.logdir, "dnf.log.1.gz")
        self.assertFile(logfile_rotated)
        with gzip.open(logfile_rotated, 'rt') as f:
            msgs = map(operator.attrgetter("message"),
                       map(_split_logfile_entry, f.readlines()))
        self.assertSequenceEqual(list(msgs), ['i'])



# -*- coding: utf-8 -*-


from __future__ import absolute_import
from __future__ import unicode_literals

import os
import unittest


REPOS_DIR = os.path.join(os.path.dirname(__file__), "../repos/")
TOUR_4_4 = os.path.join(REPOS_DIR, "rpm/tour-4-4.noarch.rpm")
COMPS = os.path.join(REPOS_DIR, "main_comps.xml")


class TestCase(unittest.TestCase):

    def _get_pkg(self):
        self.base.fill_sack(load_system_repo=False, load_available_repos=False)
        self.base.add_remote_rpms(path_list=[TOUR_4_4], strict=True, progress=None)
        pkg = self.base.sack.query().filter(name="tour")[0]
        return pkg

    def _load_comps(self):
        self.base.read_comps()
        self.base.comps._add_from_xml_filename(COMPS)

    def assertHasAttr(self, obj, name):
        self.assertTrue(hasattr(obj, name))

    def assertHasType(self, obj, types):
        self.assertTrue(isinstance(obj, types))



# -*- coding: utf-8 -*-

# Copyright (C) 2012-2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

from __future__ import absolute_import
from __future__ import unicode_literals

import rpm

import tests.support


class DistroSyncAll(tests.support.ResultTestCase):

    REPOS = ["distro"]
    INIT_SACK = True

    def test_distro_sync_all(self):
        self.base.distro_sync()
        self.assertIn(rpm.RPMPROB_FILTER_OLDPACKAGE, self.base._rpm_probfilter)
        packages = tests.support.installed_but(self.sack, "pepper", "librita").run()
        q = self.sack.query().available().filter(name=["pepper", "librita"])
        packages.extend(q)
        self.assertResult(self.base, packages)


class DistroSync(tests.support.ResultTestCase):

    REPOS = ["main", "updates"]
    BASE_CLI = True

    def test_distro_sync(self):
        installed = self._get_installed(self.base)
        original_pkg = list(filter(lambda p: p.name == "hole", installed))
        self.base.distro_sync_userlist(('bla', 'hole'))
        obsolete_pkg = list(filter(lambda p: p.name == "tour", installed))

        installed2 = self._get_installed(self.base)
        updated_pkg = list(filter(lambda p: p.name == "hole", installed2))
        self.assertLength(updated_pkg, 1)
        self.assertLength(original_pkg, 1)
        self.assertLength(updated_pkg, 1)

        # holy pkg upgraded from version 1 to 2 and obsoletes tour
        self.assertEqual(original_pkg[0].version, "1")
        self.assertEqual(updated_pkg[0].version, "2")
        installed.remove(original_pkg[0])
        installed.remove(obsolete_pkg[0])
        installed2.remove(updated_pkg[0])
        self.assertEqual(installed, installed2)



# -*- coding: utf-8 -*-


from __future__ import absolute_import
from __future__ import unicode_literals

import dnf

from .common import TestCase


class DnfSubjectApiTest(TestCase):
    def setUp(self):
        self.subject = dnf.subject.Subject("")

    def test_subject(self):
        # dnf.subject.Subject
        self.assertHasAttr(dnf.subject, "Subject")
        self.assertHasType(dnf.subject.Subject, object)

    def test_init(self):
        # Subject.__init__
        _ = dnf.subject.Subject("")

    def test_get_best_query(self):
        # Subject.get_best_query
        self.assertHasAttr(self.subject, "get_best_query")
        b = dnf.Base(dnf.conf.Conf())
        b.fill_sack(False, False)
        self.assertHasType(
            self.subject.get_best_query(
                sack=b.sack,
                with_nevra=False,
                with_provides=False,
                with_filenames=False,
                forms=None
            ), dnf.query.Query)

    def test_get_best_selector(self):
        # Subject.get_best_selector
        self.assertHasAttr(self.subject, "get_best_selector")
        b = dnf.Base(dnf.conf.Conf())
        b.fill_sack(False, False)
        self.assertHasType(
            self.subject.get_best_selector(
                sack=b.sack,
                forms=None,
                obsoletes=False,
                reponame=None
            ), dnf.selector.Selector)

    def test_get_nevra_possibilities(self):
        # Subject.get_nevra_possibilities
        self.assertHasAttr(self.subject, "get_nevra_possibilities")
        self.assertHasType(self.subject.get_nevra_possibilities(forms=None), list)



# -*- coding: utf-8 -*-

# Copyright (C) 2012-2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

"""Unit test dnf.lock module.

Locking is very hard to cover reasonably with a unit test, this is more or less
just a sanity check.

"""

from __future__ import absolute_import
from __future__ import unicode_literals

import multiprocessing
import os
import re
import threading


import dnf.lock
import dnf.pycomp
import dnf.util
from dnf.exceptions import ProcessLockError, ThreadLockError

import tests.support
from tests.support import mock


class ConcurrencyMixin(object):
    def __init__(self, lock):
        self.lock = lock

    def run(self):
        try:
            with self.lock:
                pass
        except (ProcessLockError, ThreadLockError) as e:
            self.queue.put(e)


class OtherThread(ConcurrencyMixin, threading.Thread):
    def __init__(self, lock):
        ConcurrencyMixin.__init__(self, lock)
        threading.Thread.__init__(self)
        self.queue = dnf.pycomp.Queue(1)


class OtherProcess(ConcurrencyMixin, multiprocessing.Process):
    def __init__(self, lock):
        ConcurrencyMixin.__init__(self, lock)
        multiprocessing.Process.__init__(self)
        self.queue = multiprocessing.Queue(1)


TARGET = os.path.join(tests.support.USER_RUNDIR, 'unit-test.pid')


def build_lock(blocking=False):
    return dnf.lock.ProcessLock(TARGET, 'unit-tests', blocking)


class LockTest(tests.support.TestCase):
    def test_fit_lock_dir(self):
        orig = '/some'
        with mock.patch('dnf.util.am_i_root', return_value=True):
            self.assertEqual(dnf.lock._fit_lock_dir(orig), '/some')
        with mock.patch('dnf.util.am_i_root', return_value=False):
            dir_ = dnf.lock._fit_lock_dir(orig)
            match = re.match(
                r'/var/tmp/dnf-[^:]+-[^/]+/locks/2690814ff0269c86e5109d2915ea572092918cb3',
                dir_)
            self.assertTrue(match)


class ProcessLockTest(tests.support.TestCase):
    @classmethod
    def tearDownClass(cls):
        dnf.util.rm_rf(tests.support.USER_RUNDIR)

    def test_simple(self):
        l1 = build_lock()
        target = l1.target
        with l1:
            self.assertFile(target)
        self.assertPathDoesNotExist(target)

    def test_reentrance(self):
        l1 = build_lock()
        with l1:
            with l1:
                pass

    def test_another_process(self):
        l1 = build_lock()
        process = OtherProcess(l1)
        with l1:
            process.start()
            process.join()
        self.assertIsInstance(process.queue.get(), ProcessLockError)

    def test_another_process_blocking(self):
        l1 = build_lock(blocking=True)
        l2 = build_lock(blocking=True)
        process = OtherProcess(l1)
        target = l1.target
        with l2:
            process.start()
        process.join()
        self.assertEqual(process.queue.empty(), True)
        self.assertPathDoesNotExist(target)

    def test_another_thread(self):
        l1 = build_lock()
        thread = OtherThread(l1)
        with l1:
            thread.start()
            thread.join()
        self.assertIsInstance(thread.queue.get(), ThreadLockError)



# -*- coding: utf-8 -*-

# Copyright (C) 2012-2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

from __future__ import absolute_import
from __future__ import unicode_literals

import operator

import libdnf.transaction

import dnf.comps
import dnf.util

import tests.support
from tests.support import mock


class EmptyPersistorTest(tests.support.ResultTestCase):
    """Test group operations with empty persistor."""

    REPOS = ['main']
    COMPS = True

    @mock.patch('locale.getlocale', return_value=('cs_CZ', 'UTF-8'))
    def test_group_install_locale(self, _unused):
        grp = self.comps.group_by_pattern('Kritick\xe1 cesta (Z\xe1klad)')
        cnt = self.base.group_install(grp.id, ('mandatory',))
        self.assertEqual(cnt, 2)

    def test_finalize_comps_trans(self):
        trans = dnf.comps.TransactionBunch()
        trans.install = ('trampoline',)
        self.assertGreater(self.base._add_comps_trans(trans), 0)
        self.base._finalize_comps_trans()
        self.assertIn('trampoline', self.base._goal.group_members)
        (installed, removed) = self.installed_removed(self.base)
        self.assertCountEqual(map(str, installed), ('trampoline-2.1-1.noarch',))
        self.assertEmpty(removed)


class PresetPersistorTest(tests.support.ResultTestCase):
    """Test group operations with some data in the persistor."""

    REPOS = ['main']
    COMPS = True
    COMPS_SEED_HISTORY = True

    def _install_test_env(self):
        """Env installation itself does not handle packages. We need to handle
           them manually for proper functionality of env remove"""

        env_id = 'sugar-desktop-environment'
        comps_env = self.comps._environment_by_id(env_id)
        self.base.environment_install(comps_env.id, ('mandatory',))
        self._swdb_commit()

        swdb_env = self.history.env.get(comps_env.id)
        self.assertIsNotNone(swdb_env)

        for comps_group in comps_env.mandatory_groups:
            swdb_group = self.history.group.get(comps_group.id)
            self.assertIsNotNone(swdb_group)

        tsis = []
        seen_pkgs = set()
        for swdb_env_group in swdb_env.getGroups():
            swdb_group = self.history.group.get(swdb_env_group.getGroupId())
            if not swdb_group:
                continue
            for swdb_pkg in swdb_group.getPackages():
                swdb_pkg.setInstalled(True)
                pkgs = self.base.sack.query().filter(name=swdb_pkg.getName(), arch="x86_64").run()
                if not pkgs:
                    continue
                pkg = pkgs[0]
                if pkg in seen_pkgs:
                    # prevent RPMs from being twice in a transaction and triggering unique constraint error
                    continue
                seen_pkgs.add(pkg)
                pkg._force_swdb_repoid = "main"
                self.history.rpm.add_install(pkg, reason=libdnf.transaction.TransactionItemReason_GROUP)
#                tsi = dnf.transaction.TransactionItem(
#                    dnf.transaction.INSTALL,
#                    installed=pkg,
#                    reason=libdnf.transaction.TransactionItemReason_GROUP
#                )
#                tsis.append(tsi)

        self._swdb_commit(tsis)

    def _install_test_group(self):
        """Group installation itself does not handle packages. We need to
           handle them manually for proper functionality of group remove"""
        group_id = 'somerset'
        self.base.group_install(group_id, ('mandatory',))
        swdb_group = self.history.group._installed[group_id]
        tsis = []
        for swdb_pkg in swdb_group.getPackages():
            swdb_pkg.setInstalled(True)
            pkgs = self.base.sack.query().filter(name=swdb_pkg.getName(), arch="x86_64").run()
            if not pkgs:
                continue
            pkg = pkgs[0]
            pkg._force_swdb_repoid = "main"
            self.history.rpm.add_install(pkg, reason=libdnf.transaction.TransactionItemReason_GROUP)
#            tsi = dnf.transaction.TransactionItem(
#                dnf.transaction.INSTALL,
#                installed=pkg,
#                reason=libdnf.transaction.TransactionItemReason_GROUP
#            )
#            tsis.append(tsi)

        self._swdb_commit(tsis)
        self.base.reset(goal=True)

    def test_env_group_remove(self):
        self._install_test_env()
        env_id = 'sugar-desktop-environment'
        pkg_count = self.base.env_group_remove([env_id])
        self._swdb_commit()
        self.assertEqual(3, pkg_count)
        with tests.support.mock.patch('logging.Logger.error'):
            self.assertRaises(dnf.exceptions.Error,
                              self.base.env_group_remove,
                              ['nonexistent'])

    def test_environment_remove(self):
        self._install_test_env()
        env_id = 'sugar-desktop-environment'
        swdb_env = self.history.env.get(env_id)
        self.assertIsNotNone(swdb_env)
        self.assertEqual(swdb_env.getEnvironmentId(), 'sugar-desktop-environment')

        removed_pkg_count = self.base.environment_remove(env_id)
        self.assertGreater(removed_pkg_count, 0)
        self._swdb_commit()

        swdb_env = self.history.env.get(env_id)
        self.assertIsNone(swdb_env)

        peppers = self.history.group.get('Peppers')
        self.assertIsNone(peppers)

        somerset = self.history.group.get('somerset')
        self.assertIsNone(somerset)

    def test_env_upgrade(self):
        self._install_test_env()
        cnt = self.base.environment_upgrade("sugar-desktop-environment")
        self.assertEqual(5, cnt)

        peppers = self.history.group.get('Peppers')
        self.assertIsNotNone(peppers)

        somerset = self.history.group.get('somerset')
        self.assertIsNotNone(somerset)

    def test_group_install(self):
        comps_group = self.base.comps.group_by_pattern('Base')
        pkg_count = self.base.group_install(comps_group.id, ('mandatory',))
        self.assertEqual(pkg_count, 2)
        self._swdb_commit()

        installed, removed = self.installed_removed(self.base)
        self.assertEmpty(installed)
        self.assertEmpty(removed)
        swdb_group = self.history.group.get(comps_group.id)
        self.assertIsNotNone(swdb_group)

    def test_group_remove(self):
        self._install_test_group()
        group_id = 'somerset'

        pkgs_removed = self.base.group_remove(group_id)
        self.assertGreater(pkgs_removed, 0)

        self._swdb_begin()
        installed, removed = self.installed_removed(self.base)
        self.assertEmpty(installed)
        self.assertCountEqual([pkg.name for pkg in removed], ('pepper',))
        self._swdb_end()


class ProblemGroupTest(tests.support.ResultTestCase):
    """Test some cases involving problems in groups: packages that
    don't exist, and packages that exist but cannot be installed. The
    "broken" group lists three packages. "meaning-of-life", explicitly
    'default', does not exist. "lotus", implicitly 'mandatory' (no
    explicit type), exists and is installable. "brokendeps",
    explicitly 'optional', exists but has broken dependencies. See
    https://bugzilla.redhat.com/show_bug.cgi?id=1292892,
    https://bugzilla.redhat.com/show_bug.cgi?id=1337731,
    https://bugzilla.redhat.com/show_bug.cgi?id=1427365, and
    https://bugzilla.redhat.com/show_bug.cgi?id=1461539 for some of
    the background on this.
    """

    REPOS = ['main', 'broken_group']
    COMPS = True
    COMPS_SEED_PERSISTOR = True

    def test_group_install_broken_mandatory(self):
        """Here we will test installing the group with only mandatory
        packages. We expect this to succeed, leaving out the
        non-existent 'meaning-of-life': it should also log a warning,
        but we don't test that.
        """
        comps_group = self.base.comps.group_by_pattern('Broken Group')
        swdb_group = self.history.group.get(comps_group.id)
        self.assertIsNone(swdb_group)

        cnt = self.base.group_install(comps_group.id, ('mandatory',))
        self._swdb_commit()
        self.base.resolve()
        # this counts packages *listed* in the group, so 2
        self.assertEqual(cnt, 2)

        inst, removed = self.installed_removed(self.base)
        # the above should work, but only 'lotus' actually installed
        self.assertLength(inst, 1)
        self.assertEmpty(removed)

    def test_group_install_broken_default(self):
        """Here we will test installing the group with only mandatory
        and default packages. Again we expect this to succeed: the new
        factor is an entry pulling in librita if no-such-package is
        also included or installed. We expect this not to actually
        pull in librita (as no-such-package obviously *isn't* there),
        but also not to cause a fatal error.
        """
        comps_group = self.base.comps.group_by_pattern('Broken Group')
        swdb_group = self.history.group.get(comps_group.id)
        self.assertIsNone(swdb_group)

        cnt = self.base.group_install(comps_group.id, ('mandatory', 'default'))
        self._swdb_commit()
        self.base.resolve()
        # this counts packages *listed* in the group, so 3
        self.assertEqual(cnt, 3)

        inst, removed = self.installed_removed(self.base)
        # the above should work, but only 'lotus' actually installed
        self.assertLength(inst, 1)
        self.assertEmpty(removed)

    def test_group_install_broken_optional(self):
        """Here we test installing the group with optional packages
        included. We expect this to fail, as a package that exists but
        has broken dependencies is now included.
        """
        comps_group = self.base.comps.group_by_pattern('Broken Group')
        swdb_group = self.history.group.get(comps_group.id)
        self.assertIsNone(swdb_group)

        cnt = self.base.group_install(comps_group.id, ('mandatory', 'default', 'optional'))
        self.assertEqual(cnt, 4)

        self._swdb_commit()
        # this should fail, as optional 'brokendeps' is now pulled in
        self.assertRaises(dnf.exceptions.DepsolveError, self.base.resolve)

    def test_group_install_broken_optional_nonstrict(self):
        """Here we test installing the group with optional packages
        included, but with strict=False. We expect this to succeed,
        skipping the package with broken dependencies.
        """
        comps_group = self.base.comps.group_by_pattern('Broken Group')
        swdb_group = self.history.group.get(comps_group.id)
        self.assertIsNone(swdb_group)

        cnt = self.base.group_install(comps_group.id, ('mandatory', 'default', 'optional'),
                                      strict=False)
        self._swdb_commit()
        self.base.resolve()
        self.assertEqual(cnt, 4)

        inst, removed = self.installed_removed(self.base)
        # the above should work, but only 'lotus' actually installed
        self.assertLength(inst, 1)
        self.assertEmpty(removed)

    def test_group_install_missing_name(self):
        comps_group = self.base.comps.group_by_pattern('missing-name-group')

        cnt = self.base.group_install(comps_group.id, ('mandatory', 'default', 'optional'),
                                      strict=False)
        self._swdb_commit()
        self.base.resolve()
        self.assertEqual(cnt, 1)


class EnvironmentInstallTest(tests.support.ResultTestCase):
    """Set up a test where sugar is considered not installed."""

    REPOS = ['main']
    COMPS = True
    COMPS_SEED_HISTORY = True

    def test_environment_install(self):
        env_id = 'sugar-desktop-environment'
        comps_env = self.comps.environment_by_pattern(env_id)
        self.base.environment_install(comps_env.id, ('mandatory',))
        self._swdb_commit()

        installed, _ = self.installed_removed(self.base)
        self.assertCountEqual(map(operator.attrgetter('name'), installed),
                              ('trampoline', 'lotus'))

        swdb_env = self.history.env.get(env_id)
        self.assertCountEqual([i.getGroupId() for i in swdb_env.getGroups()], ('somerset', 'Peppers', 'base'))

        peppers = self.history.group.get('Peppers')
        self.assertIsNotNone(peppers)

        somerset = self.history.group.get('somerset')
        self.assertIsNotNone(somerset)

        base = self.history.group.get('base')
        self.assertIsNone(base)



# -*- coding: utf-8 -*-


from __future__ import absolute_import
from __future__ import unicode_literals

import dnf

from .common import TestCase


class DnfTransactionDisplayApiTest(TestCase):
    def setUp(self):
        self.td = dnf.yum.rpmtrans.TransactionDisplay()

    def test_init(self):
        td = dnf.yum.rpmtrans.TransactionDisplay()
        self.assertHasType(td, dnf.yum.rpmtrans.TransactionDisplay)

    def test_progress(self):
        # TransactionDisplay.progress
        self.assertHasAttr(self.td, "progress")
        self.td.progress(
            package=None,
            action=None,
            ti_done=None,
            ti_total=None,
            ts_done=None,
            ts_total=None
        )

    def test_error(self):
        # RPMTransaction.error
        self.assertHasAttr(self.td, "error")
        self.td.error(message="")



# -*- coding: utf-8 -*-


from __future__ import absolute_import
from __future__ import unicode_literals

import dnf
import dnf.conf

from .common import TestCase
from .common import TOUR_4_4


class DnfBaseApiTest(TestCase):
    def setUp(self):
        self.base = dnf.Base(dnf.conf.Conf())

    def tearDown(self):
        self.base.close()

    def test_base(self):
        # dnf.base.Base
        self.assertHasAttr(dnf.base, "Base")
        self.assertHasType(dnf.base.Base, object)

    def test_init(self):
        base = dnf.Base(dnf.conf.Conf())

    def test_init_conf(self):
        conf = dnf.conf.Conf()
        base = dnf.base.Base(conf=conf)

    def test_comps(self):
        # Base.comps
        self.assertHasAttr(self.base, "comps")

        # blank initially
        self.assertEqual(self.base.comps, None)

        self.base.read_comps()
        self.assertHasType(self.base.comps, dnf.comps.Comps)

    def test_conf(self):
        # Base.conf
        self.assertHasAttr(self.base, "conf")
        self.assertHasType(self.base.conf, dnf.conf.Conf)

    def test_repos(self):
        # Base.repos
        self.assertHasAttr(self.base, "repos")
        self.assertHasType(self.base.repos, dnf.repodict.RepoDict)

        del self.base.repos
        self.assertEqual(self.base.repos, None)

    def test_sack(self):
        # Base.sack
        self.assertHasAttr(self.base, "sack")

        # blank initially
        self.assertEqual(self.base.sack, None)

        self.base.fill_sack(False, False)
        self.assertHasType(self.base.sack, dnf.sack.Sack)

    def test_transaction(self):
        # Base.transaction
        self.assertHasAttr(self.base, "transaction")

        # blank initially
        self.assertEqual(self.base.transaction, None)

        # transaction attribute is set after resolving a transaction
        self.base.fill_sack(False, False)
        self.base.resolve()
        self.assertHasType(self.base.transaction, dnf.db.group.RPMTransaction)

    def test_init_plugins(self):
        # Base.init_plugins(disabled_glob=(), enable_plugins=(), cli=None)
        self.assertHasAttr(self.base, "init_plugins")

        # disable plugins to avoid calling dnf.plugin.Plugins._load() multiple times
        # which causes the tests to crash
        self.base.conf.plugins = False
        self.base.init_plugins(disabled_glob=(), enable_plugins=(), cli=None)

    def test_pre_configure_plugins(self):
        # Base.pre_configure_plugins()
        self.assertHasAttr(self.base, "pre_configure_plugins")

        self.base.pre_configure_plugins()

    def test_configure_plugins(self):
        # Base.configure_plugins()
        self.assertHasAttr(self.base, "configure_plugins")

        self.base.configure_plugins()

    def test_update_cache(self):
        # Base.update_cache(self, timer=False)
        self.assertHasAttr(self.base, "update_cache")

        self.base.update_cache(timer=False)

    def test_fill_sack(self):
        # Base.fill_sack(self, load_system_repo=True, load_available_repos=True):
        self.assertHasAttr(self.base, "fill_sack")

        self.base.fill_sack(load_system_repo=False, load_available_repos=False)

    def test_close(self):
        # Base.close()
        self.assertHasAttr(self.base, "close")

        self.base.close()

    def test_read_all_repos(self):
        # Base.read_all_repos(self, opts=None):
        self.assertHasAttr(self.base, "read_all_repos")

        self.base.read_all_repos(opts=None)

    def test_reset(self):
        # Base.reset(self, sack=False, repos=False, goal=False):cloread_all_repos(self, opts=None)
        self.assertHasAttr(self.base, "reset")

        self.base.reset(sack=False, repos=False, goal=False)

    def test_read_comps(self):
        # Base.read_comps(self, arch_filter=False)
        self.assertHasAttr(self.base, "read_comps")

        self.base.read_comps(arch_filter=False)

    def test_resolve(self):
        # Base.resolve(self, allow_erasing=False)
        self.assertHasAttr(self.base, "resolve")

        self.base.fill_sack(load_system_repo=False, load_available_repos=False)
        self.base.resolve(allow_erasing=False)

    def test_do_transaction(self):
        # Base.do_transaction(self, display=())
        self.assertHasAttr(self.base, "do_transaction")

        self.base.fill_sack(load_system_repo=False, load_available_repos=False)
        self.base.resolve(allow_erasing=False)
        self.base.do_transaction(display=None)

    def test_download_packages(self):
        # Base.download_packages(self, pkglist, progress=None, callback_total=None)
        self.assertHasAttr(self.base, "download_packages")

        self.base.download_packages(pkglist=[], progress=None, callback_total=None)

    def test_add_remote_rpms(self):
        # Base.add_remote_rpms(self, path_list, strict=True, progress=None)
        self.assertHasAttr(self.base, "add_remote_rpms")

        self.base.fill_sack(load_system_repo=False, load_available_repos=False)
        self.base.add_remote_rpms(path_list=[TOUR_4_4], strict=True, progress=None)

    def test_package_signature_check(self):
        # Base.package_signature_check(self, pkg)
        self.assertHasAttr(self.base, "package_signature_check")
        self.base.package_signature_check(pkg=self._get_pkg())

    def test_package_import_key(self):
        # Base.package_import_key(self, pkg, askcb=None, fullaskcb=None)
        self.assertHasAttr(self.base, "package_import_key")
        self.assertRaises(
            ValueError,
            self.base.package_import_key,
            pkg=self._get_pkg(),
            askcb=None,
            fullaskcb=None,
        )

    def test_environment_install(self):
        # Base.environment_install(self, env_id, types, exclude=None, strict=True, exclude_groups=None)
        self.assertHasAttr(self.base, "environment_install")
        self._load_comps()
        self.base.environment_install(
            env_id="sugar-desktop-environment",
            types=["mandatory", "default", "optional"],
            exclude=None,
            strict=True,
            exclude_groups=None
        )

    def test_environment_remove(self):
        # Base.environment_remove(self, env_id):
        self.assertHasAttr(self.base, "environment_remove")

        self.base.read_comps(arch_filter=False)
        self.assertRaises(dnf.exceptions.CompsError, self.base.environment_remove, env_id="base")

    def test_environment_upgrade(self):
        # Base.environment_upgrade(self, env_id):
        self.assertHasAttr(self.base, "environment_upgrade")

        self._load_comps()
        self.assertRaises(dnf.exceptions.CompsError, self.base.environment_upgrade, env_id="sugar-desktop-environment")

    def test_group_install(self):
        # Base.group_install(self, grp_id, pkg_types, exclude=None, strict=True)
        self.assertHasAttr(self.base, "group_install")

        self._load_comps()
        self.base.group_install(
            grp_id="base",
            pkg_types=["mandatory", "default", "optional"],
            exclude=None,
            strict=True
        )

    def test_group_remove(self):
        # Base.group_remove(self, env_id):
        self.assertHasAttr(self.base, "group_remove")

        self._load_comps()
        self.assertRaises(dnf.exceptions.CompsError, self.base.group_remove, grp_id="base")

    def test_group_upgrade(self):
        # Base.group_upgrade(self, env_id):
        self.assertHasAttr(self.base, "group_upgrade")

        self.base.read_comps(arch_filter=False)
        self.assertRaises(dnf.exceptions.CompsError, self.base.group_upgrade, grp_id="base")

    def test_install_specs(self):
        # Base.install_specs(self, install, exclude=None, reponame=None, strict=True, forms=None)
        self.base.fill_sack(load_system_repo=False, load_available_repos=False)
        self.base.install_specs(install=[], exclude=None, reponame=None, strict=True, forms=None)

    def test_install(self):
        # Base.install(self, pkg_spec, reponame=None, strict=True, forms=None)
        self.base.fill_sack(load_system_repo=False, load_available_repos=False)
        self.assertRaises(
            dnf.exceptions.PackageNotFoundError,
            self.base.install,
            pkg_spec="",
            reponame=None,
            strict=True,
            forms=None
        )

    def test_package_downgrade(self):
        # Base.package_downgrade(self, pkg, strict=False)
        pkg = self._get_pkg()
        self.assertRaises(dnf.exceptions.MarkingError, self.base.package_downgrade, pkg=pkg, strict=False)

    def test_package_install(self):
        # Base.package_install(self, pkg, strict=False)
        pkg = self._get_pkg()
        self.base.package_install(pkg=pkg, strict=False)

    def test_package_upgrade(self):
        # Base.package_upgrade(self, pkg)
        pkg = self._get_pkg()
        self.assertRaises(dnf.exceptions.MarkingError, self.base.package_upgrade, pkg=pkg)

    def test_upgrade(self):
        # Base.upgrade(self, pkg_spec, reponame=None)
        self.base.fill_sack(load_system_repo=False, load_available_repos=False)
        self.assertRaises(dnf.exceptions.MarkingError, self.base.upgrade, pkg_spec="", reponame=None)

    def test_upgrade_all(self):
        # Base.upgrade_all(self, reponame=None)
        self.base.fill_sack(load_system_repo=False, load_available_repos=False)
        self.base.upgrade_all(reponame=None)

    def test_autoremove(self):
        # Base.autoremove(self, forms=None, pkg_specs=None, grp_specs=None, filenames=None)
        self.base.fill_sack(load_system_repo=False, load_available_repos=False)
        self.base.autoremove(forms=None, pkg_specs=None, grp_specs=None, filenames=None)

    def test_remove(self):
        # Base.remove(self, pkg_spec, reponame=None, forms=None)
        self.base.fill_sack(load_system_repo=False, load_available_repos=False)
        self.assertRaises(dnf.exceptions.MarkingError, self.base.remove, pkg_spec="", reponame=None, forms=None)

    def test_downgrade(self):
        # Base.downgrade(self, pkg_spec)
        self.base.fill_sack(load_system_repo=False, load_available_repos=False)
        self.assertRaises(dnf.exceptions.MarkingError, self.base.downgrade, pkg_spec="")

    def test_urlopen(self):
        # Base.urlopen(self, url, repo=None, mode='w+b', **kwargs)
        self.base.urlopen(url="file:///dev/null", repo=None, mode='w+b')

    def test_setup_loggers(self):
        # Base.setup_loggers(self)
        self.base.setup_loggers()



# -*- coding: utf-8 -*-


from __future__ import absolute_import
from __future__ import unicode_literals

import dnf

import logging
from .common import TestCase


class DnfLoggingApiTest(TestCase):
    def test_levels(self):
        self.assertHasAttr(dnf.logging, "SUPERCRITICAL")
        self.assertHasType(dnf.logging.SUPERCRITICAL, int)

        self.assertHasAttr(dnf.logging, "CRITICAL")
        self.assertHasType(dnf.logging.CRITICAL, int)

        self.assertHasAttr(dnf.logging, "ERROR")
        self.assertHasType(dnf.logging.ERROR, int)

        self.assertHasAttr(dnf.logging, "WARNING")
        self.assertHasType(dnf.logging.WARNING, int)

        self.assertHasAttr(dnf.logging, "INFO")
        self.assertHasType(dnf.logging.INFO, int)

        self.assertHasAttr(dnf.logging, "DEBUG")
        self.assertHasType(dnf.logging.DEBUG, int)

        self.assertHasAttr(dnf.logging, "DDEBUG")
        self.assertHasType(dnf.logging.DDEBUG, int)

        self.assertHasAttr(dnf.logging, "SUBDEBUG")
        self.assertHasType(dnf.logging.SUBDEBUG, int)

        self.assertHasAttr(dnf.logging, "TRACE")
        self.assertHasType(dnf.logging.TRACE, int)

        self.assertHasAttr(dnf.logging, "ALL")
        self.assertHasType(dnf.logging.ALL, int)

    def test_logging_level_names(self):
        # Level names added in dnf logging initialization
        self.assertTrue(logging.getLevelName(dnf.logging.DDEBUG) == "DDEBUG")
        self.assertTrue(logging.getLevelName(dnf.logging.SUBDEBUG) == "SUBDEBUG")
        self.assertTrue(logging.getLevelName(dnf.logging.TRACE) == "TRACE")

    def test_dnf_logger(self):
        # This doesn't really test much since python allows getting any logger,
        # at least check if it has handlers (setup in dnf).
        logger = logging.getLogger('dnf')
        self.assertTrue(len(logger.handlers) > 0)

    def test_dnf_rpm_logger(self):
        # This doesn't really test dnf api since python allows getting any logger,
        # but at least check that the messages are somehow taken care of.
        logger = logging.getLogger('dnf.rpm')
        self.assertTrue(len(logger.handlers) > 0 or logger.propagate)

    def test_dnf_plugin_logger(self):
        # This doesn't really test dnf api since python allows getting any logger,
        # but at least check that the messages are somehow taken care of.
        logger = logging.getLogger('dnf.plugin')
        self.assertTrue(len(logger.handlers) > 0 or logger.propagate)



# Copyright (C) 2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

from __future__ import absolute_import
from __future__ import unicode_literals

import unittest

import dnf


class ListOptionTest(unittest.TestCase):

    def setUp(self):
        self.conf = dnf.conf.MainConf()

    def test_delitem(self):
        self.conf.pluginpath = ["a", "b", "c"]
        self.assertEqual(list(self.conf.pluginpath), ["a", "b", "c"])

        var = self.conf.pluginpath
        self.assertEqual(var, ["a", "b", "c"])

        del var[0]
        self.assertEqual(list(self.conf.pluginpath), ["b", "c"])
        self.assertEqual(var, ["b", "c"])

    def test_iadd(self):
        self.conf.pluginpath = ["a"]
        self.assertEqual(self.conf.pluginpath, ["a"])

        var = self.conf.pluginpath
        self.assertEqual(var, ["a"])

        var += ["b", "c"]
        self.assertEqual(self.conf.pluginpath, ["a", "b", "c"])
        self.assertEqual(var, ["a", "b", "c"])

    def test_iadd_tuple(self):
        self.conf.pluginpath = ("a", )
        self.assertEqual(self.conf.pluginpath, ["a"])

        var = self.conf.pluginpath
        self.assertEqual(var, ["a"])

        var += ("b", "c")
        self.assertEqual(self.conf.pluginpath, ["a", "b", "c"])
        self.assertEqual(var, ["a", "b", "c"])

    def test_imul(self):
        self.conf.pluginpath = ["a"]
        self.assertEqual(self.conf.pluginpath, ["a"])

        var = self.conf.pluginpath
        self.assertEqual(var, ["a"])

        var *= 3
        self.assertEqual(self.conf.pluginpath, ["a", "a", "a"])
        self.assertEqual(var, ["a", "a", "a"])

    def test_setitem(self):
        self.conf.pluginpath = ["a"]
        self.assertEqual(self.conf.pluginpath, ["a"])

        var = self.conf.pluginpath
        self.assertEqual(var, ["a"])

        var[0] = "b"
        self.assertEqual(self.conf.pluginpath, ["b"])
        self.assertEqual(var, ["b"])

    def test_append(self):
        self.conf.pluginpath = ["a"]
        self.assertEqual(self.conf.pluginpath, ["a"])

        var = self.conf.pluginpath
        self.assertEqual(var, ["a"])

        var.append("b")
        self.assertEqual(self.conf.pluginpath, ["a", "b"])
        self.assertEqual(var, ["a", "b"])

    def test_clear(self):
        self.conf.pluginpath = ["a"]
        self.assertEqual(self.conf.pluginpath, ["a"])

        var = self.conf.pluginpath
        self.assertEqual(var, ["a"])

        var.clear()
        self.assertEqual(self.conf.pluginpath, [])
        self.assertEqual(var, [])

    def test_extend(self):
        self.conf.pluginpath = ["a"]
        self.assertEqual(self.conf.pluginpath, ["a"])

        var = self.conf.pluginpath
        self.assertEqual(var, ["a"])

        var.extend(["b", "c"])
        self.assertEqual(self.conf.pluginpath, ["a", "b", "c"])
        self.assertEqual(var, ["a", "b", "c"])

    def test_insert(self):
        self.conf.pluginpath = ["a"]
        self.assertEqual(self.conf.pluginpath, ["a"])

        var = self.conf.pluginpath
        self.assertEqual(var, ["a"])

        var.insert(0, "b")
        self.assertEqual(self.conf.pluginpath, ["b", "a"])
        self.assertEqual(var, ["b", "a"])

    def test_pop(self):
        self.conf.pluginpath = ["a", "b", "c"]
        self.assertEqual(self.conf.pluginpath, ["a", "b", "c"])

        var = self.conf.pluginpath
        self.assertEqual(var, ["a", "b", "c"])

        var.pop()
        self.assertEqual(self.conf.pluginpath, ["a", "b"])
        self.assertEqual(var, ["a", "b"])

    def test_remove(self):
        self.conf.pluginpath = ["a", "b", "c"]
        self.assertEqual(self.conf.pluginpath, ["a", "b", "c"])

        var = self.conf.pluginpath
        self.assertEqual(var, ["a", "b", "c"])

        var.remove("b")
        self.assertEqual(self.conf.pluginpath, ["a", "c"])
        self.assertEqual(var, ["a", "c"])

    def test_reverse(self):
        self.conf.pluginpath = ["a", "b", "c"]
        self.assertEqual(self.conf.pluginpath, ["a", "b", "c"])

        var = self.conf.pluginpath
        self.assertEqual(var, ["a", "b", "c"])

        var.reverse()
        self.assertEqual(self.conf.pluginpath, ["c", "b", "a"])
        self.assertEqual(var, ["c", "b", "a"])

    def test_sort(self):
        self.conf.pluginpath = ["a", "c", "b"]
        self.assertEqual(self.conf.pluginpath, ["a", "c", "b"])

        var = self.conf.pluginpath
        self.assertEqual(var, ["a", "c", "b"])

        var.sort()
        self.assertEqual(self.conf.pluginpath, ["a", "b", "c"])
        self.assertEqual(var, ["a", "b", "c"])

    def test_priority(self):
        self.conf._set_value("pluginpath", ["a"], dnf.conf.config.PRIO_PLUGINDEFAULT)
        self.assertEqual(self.conf.pluginpath, ["a"])

        self.conf._set_value("pluginpath", ["b"], dnf.conf.config.PRIO_PLUGINDEFAULT)
        self.assertEqual(self.conf.pluginpath, ["b"])

        # setting with lower priority is NOT allowed
        self.conf._set_value("pluginpath", ["c"], dnf.conf.config.PRIO_DEFAULT)
        self.assertEqual(self.conf.pluginpath, ["b"])

        # resetting with lower priority is NOT allowed
        self.conf._set_value("pluginpath", [], dnf.conf.config.PRIO_DEFAULT)
        self.assertEqual(self.conf.pluginpath, ["b"])

        self.conf.pluginpath = ["d"]
        self.assertEqual(self.conf.pluginpath, ["d"])

        self.conf.pluginpath.append("e")
        self.assertEqual(self.conf.pluginpath, ["d", "e"])

        self.conf.pluginpath = []
        self.assertEqual(self.conf.pluginpath, [])

    def test_references(self):
        self.conf.pluginpath = ["a"]
        self.assertEqual(list(self.conf.pluginpath), ["a"])

        var1 = self.conf.pluginpath
        var2 = self.conf.pluginpath
        self.assertEqual(var1, ["a"])
        self.assertEqual(var2, ["a"])

        var1 += ["b", "c"]
        self.assertEqual(list(self.conf.pluginpath), ["a", "b", "c"])
        self.assertEqual(var1, ["a", "b", "c"])
        self.assertEqual(var2, ["a", "b", "c"])

    def test_add(self):
        self.conf.pluginpath = ["a"]
        self.assertEqual(list(self.conf.pluginpath), ["a"])

        var = self.conf.pluginpath
        self.conf._set_value("pluginpath", ["b"])
        var = var + ["c"]
        self.assertEqual(list(self.conf.pluginpath), ["b"])
        self.assertEqual(var, ["b", "c"])

    def test_contains(self):
        self.conf.pluginpath = ["a"]
        self.assertEqual(list(self.conf.pluginpath), ["a"])

        var = self.conf.pluginpath
        self.conf._set_value("pluginpath", ["b"])
        self.assertIn("b", var)

    def test_eq(self):
        self.conf.pluginpath = ["a"]
        self.assertEqual(list(self.conf.pluginpath), ["a"])

        var = self.conf.pluginpath
        self.conf._set_value("pluginpath", ["b"])
        self.assertEqual(var, ["b"])

    def test_ge(self):
        self.conf.pluginpath = ["a"]
        self.assertEqual(list(self.conf.pluginpath), ["a"])

        var = self.conf.pluginpath
        self.conf._set_value("pluginpath", ["b"])
        self.assertTrue(var >= ["a", "b"])

    def test_gt(self):
        self.conf.pluginpath = ["a"]
        self.assertEqual(list(self.conf.pluginpath), ["a"])

        var = self.conf.pluginpath
        self.conf._set_value("pluginpath", ["b"])
        self.assertTrue(var > ["a", "a"])

    def test_le(self):
        self.conf.pluginpath = ["a", "b"]
        self.assertEqual(list(self.conf.pluginpath), ["a", "b"])

        var = self.conf.pluginpath
        self.conf._set_value("pluginpath", [])
        self.conf._set_value("pluginpath", ["a"])
        self.assertTrue(var <= ["a"])

    def test_lt(self):
        self.conf.pluginpath = ["a", "b"]
        self.assertEqual(list(self.conf.pluginpath), ["a", "b"])

        var = self.conf.pluginpath
        self.conf._set_value("pluginpath", [])
        self.conf._set_value("pluginpath", ["a"])
        self.assertTrue(var < ["b"])

    def test_ne(self):
        self.conf.pluginpath = ["a"]
        self.assertEqual(list(self.conf.pluginpath), ["a"])

        var = self.conf.pluginpath
        self.conf._set_value("pluginpath", ["b"])
        self.assertNotEqual(var, ["a"])

    def test_mul(self):
        self.conf.pluginpath = ["a"]
        self.assertEqual(list(self.conf.pluginpath), ["a"])

        var = self.conf.pluginpath
        self.assertEqual(var, ["a"])

        var = var * 3
        self.assertEqual(list(self.conf.pluginpath), ["a"])
        self.assertEqual(var, ["a", "a", "a"])

    def test_rmul(self):
        self.conf.pluginpath = ["a"]
        self.assertEqual(list(self.conf.pluginpath), ["a"])

        var = self.conf.pluginpath
        self.assertEqual(var, ["a"])

        var = 3 * var
        self.assertEqual(list(self.conf.pluginpath), ["a"])
        self.assertEqual(var, ["a", "a", "a"])

    def test_getitem(self):
        self.conf.pluginpath = ["a"]
        self.assertEqual(list(self.conf.pluginpath), ["a"])

        var = self.conf.pluginpath
        self.conf._set_value("pluginpath", ["a", "b"])
        self.assertEqual(var[1], "b")

    def test_iter(self):
        self.conf.pluginpath = ["a"]
        self.assertEqual(list(self.conf.pluginpath), ["a"])

        var = self.conf.pluginpath
        self.conf._set_value("pluginpath", ["b"])

        length = 0
        for i in iter(var):
            length += 1
            if length == 1:
                self.assertEqual(i, "b")
        self.assertEqual(length, 1)

    def test_len(self):
        self.conf.pluginpath = ["a", "b"]
        self.assertEqual(list(self.conf.pluginpath), ["a", "b"])

        var = self.conf.pluginpath
        self.conf._set_value("pluginpath", ["b"])
        self.assertEqual(len(var), 1)

    def test_copy(self):
        self.conf.pluginpath = ["a"]
        self.assertEqual(list(self.conf.pluginpath), ["a"])

        var = self.conf.pluginpath
        self.conf._set_value("pluginpath", ["b"])
        var2 = var.copy()
        self.assertEqual(var2, ["b"])

    def test_count(self):
        self.conf.pluginpath = ["a"]
        self.assertEqual(list(self.conf.pluginpath), ["a"])

        var = self.conf.pluginpath
        self.conf._set_value("pluginpath", ["a", "a"])
        self.assertEqual(var.count("a"), 2)

    def test_index(self):
        self.conf.pluginpath = ["a"]
        self.assertEqual(list(self.conf.pluginpath), ["a"])

        var = self.conf.pluginpath
        self.conf._set_value("pluginpath", ["a", "b"])
        self.assertEqual(var.index("b"), 1)



# -*- coding: utf-8 -*-


from __future__ import absolute_import
from __future__ import unicode_literals

import dnf
import libdnf

from .common import TestCase
from .common import TOUR_4_4


class DnfPluginApiTest(TestCase):
    def setUp(self):
        self.plugin = dnf.Plugin(base=None, cli=None)

    def test_plugin(self):
        # dnf.Plugin
        self.assertHasAttr(dnf, "Plugin")
        self.assertHasType(dnf.Plugin, object)

    def test_name(self):
        # Plugin.name
        self.assertHasAttr(self.plugin, "name")
        self.plugin.name = "test"
        self.assertHasType(self.plugin.name, str)

    def test_read_config(self):
        # dnf.Plugin.read_config
        self.assertHasAttr(dnf.Plugin, "read_config")
        self.assertHasType(dnf.Plugin.read_config(conf=dnf.conf.Conf()), libdnf.conf.ConfigParser)

    def test_init(self):
        # Plugin.__init__
        _ = dnf.Plugin(base=None, cli=None)

    def test_pre_config(self):
        # Plugin.pre_config
        self.assertHasAttr(self.plugin, "pre_config")
        self.plugin.pre_config()

    def test_config(self):
        # Plugin.config
        self.assertHasAttr(self.plugin, "config")
        self.plugin.config()

    def test_resolved(self):
        # Plugin.resolved
        self.assertHasAttr(self.plugin, "resolved")
        self.plugin.resolved()

    def test_sack(self):
        # Plugin.sack
        self.assertHasAttr(self.plugin, "sack")
        self.plugin.sack()

    def test_pre_transaction(self):
        # Plugin.pre_transaction
        self.assertHasAttr(self.plugin, "pre_transaction")
        self.plugin.pre_transaction()

    def test_transaction(self):
        # Plugin.transaction
        self.assertHasAttr(self.plugin, "transaction")
        self.plugin.transaction()


class Dnfregister_commandApiTest(TestCase):
    def test_register_command(self):
        self.assertHasAttr(dnf.plugin, "register_command")

        @dnf.plugin.register_command
        class TestClassWithDecorator():
            aliases = ('necessary-alias',)

        self.assertHasAttr(TestClassWithDecorator, "_plugin")



# -*- coding: utf-8 -*-

# Copyright (C) 2012-2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

from __future__ import absolute_import
from __future__ import unicode_literals

import itertools

import libdnf.transaction

import dnf.cli.commands

import tests.support


class Remove(tests.support.ResultTestCase):

    REPOS = []

    def setUp(self):
        super(Remove, self).setUp()
        self.allow_erasing = True

    def test_not_installed(self):
        """ Removing a not-installed package is a void operation. """
        with self.assertRaises(dnf.exceptions.PackagesNotInstalledError) as context:
            self.base.remove('mrkite')
        self.assertEqual(context.exception.pkg_spec, 'mrkite')
        installed_pkgs = self.base.sack.query().installed().run()
        self.assertResult(self.base, installed_pkgs)

    def test_remove(self):
        """ Simple remove. """
        self.base.remove("pepper")
        self.assertResult(self.base,
                          tests.support.installed_but(self.base.sack, "pepper"))

    def test_remove_dependent(self):
        """ Remove a lib that some other package depends on. """
        self.base.remove("librita")
        # we should end up with nothing in this case:
        new_set = tests.support.installed_but(self.base.sack, "librita", "pepper")
        self.assertResult(self.base, new_set)

    def test_remove_nevra(self):
        self.base.remove("pepper-20-0.x86_64")
        pepper = self.base.sack.query().installed().filter(name="pepper")
        (installed, removed) = self.installed_removed(self.base)
        self.assertLength(installed, 0)
        self.assertCountEqual(removed, pepper.run())

    def test_remove_glob(self):
        """ Test that weird input combinations with globs work. """
        ret = self.base.remove("*.i686")
        self.assertEqual(ret, 1)

    def test_remove_provides(self):
        """Remove uses provides too."""
        self.assertEqual(1, self.base.remove('parking'))

    def test_reponame(self):
        """Test whether only packages from the repository are uninstalled."""
        pkg_subj = dnf.subject.Subject('librita.x86_64')

        tsis = []
        for pkg in pkg_subj.get_best_query(self.base.sack).installed():
            pkg._force_swdb_repoid = "main"
            self.history.rpm.add_install(pkg)
#            tsi = dnf.transaction.TransactionItem(
#                dnf.transaction.INSTALL,
#                installed=pkg,
#                reason=libdnf.transaction.TransactionItemReason_USER
#            )
#            tsis.append(tsi)
        self._swdb_commit(tsis)

        self.base.remove('librita', 'main')
        self.assertResult(self.base, itertools.chain(
            self.base.sack.query().installed().filter(name__neq='librita'),
            dnf.subject.Subject('librita.i686').get_best_query(self.base.sack).installed())
        )



# -*- coding: utf-8 -*-


from __future__ import absolute_import
from __future__ import unicode_literals

import os

import dnf
import libdnf

from .common import TestCase


class DnfModulePackageApiTest(TestCase):
    def setUp(self):
        self.base = dnf.Base(dnf.conf.Conf())
        repo = self.base.repos.add_new_repo(
            'api-module-test-repo', self.base.conf,
            baseurl=[os.path.join(os.path.dirname(__file__), "../modules/modules/_all/x86_64/")]
        )
        self.base.fill_sack(load_system_repo=False, load_available_repos=True)
        moduleBase = dnf.module.module_base.ModuleBase(self.base)
        modulePackages, nsvcap = moduleBase.get_modules('*')
        self.modulePackage = modulePackages[0]

    def tearDown(self):
        self.base.close()

    def test_getName(self):
        # ModulePackage.getName()
        self.assertHasAttr(self.modulePackage, "getName")
        self.modulePackage.getName()

    def test_getStream(self):
        # ModulePackage.getStream()
        self.assertHasAttr(self.modulePackage, "getStream")
        self.modulePackage.getStream()

    def test_getVersion(self):
        # ModulePackage.getVersion()
        self.assertHasAttr(self.modulePackage, "getVersion")
        self.modulePackage.getVersion()

    def test_getVersionNum(self):
        # ModulePackage.getVersionNum()
        self.assertHasAttr(self.modulePackage, "getVersionNum")
        self.modulePackage.getVersionNum()

    def test_getContext(self):
        # ModulePackage.getContext()
        self.assertHasAttr(self.modulePackage, "getContext")
        self.modulePackage.getContext()

    def test_getArch(self):
        # ModulePackage.getArch()
        self.assertHasAttr(self.modulePackage, "getArch")
        self.modulePackage.getArch()

    def test_getNameStream(self):
        # ModulePackage.getNameStream()
        self.assertHasAttr(self.modulePackage, "getNameStream")
        self.modulePackage.getNameStream()

    def test_getNameStreamVersion(self):
        # ModulePackage.getNameStreamVersion()
        self.assertHasAttr(self.modulePackage, "getNameStreamVersion")
        self.modulePackage.getNameStreamVersion()

    def test_getFullIdentifier(self):
        # ModulePackage.getFullIdentifier()
        self.assertHasAttr(self.modulePackage, "getFullIdentifier")
        self.modulePackage.getFullIdentifier()

    def test_getProfiles(self):
        # ModulePackage.getProfiles()
        self.assertHasAttr(self.modulePackage, "getProfiles")
        self.modulePackage.getProfiles("test_name_argument")

    def test_getSummary(self):
        # ModulePackage.getSummary()
        self.assertHasAttr(self.modulePackage, "getSummary")
        self.modulePackage.getSummary()

    def test_getDescription(self):
        # ModulePackage.getDescription()
        self.assertHasAttr(self.modulePackage, "getDescription")
        self.modulePackage.getDescription()

    def test_getRepoID(self):
        # ModulePackage.getRepoID()
        self.assertHasAttr(self.modulePackage, "getRepoID")
        self.modulePackage.getRepoID()

    def test_getArtifacts(self):
        # ModulePackage.getArtifacts()
        self.assertHasAttr(self.modulePackage, "getArtifacts")
        self.modulePackage.getArtifacts()

    def test_getModuleDependencies(self):
        # ModulePackage.getModuleDependencies()
        self.assertHasAttr(self.modulePackage, "getModuleDependencies")
        self.modulePackage.getModuleDependencies()

    def test_getYaml(self):
        # ModulePackage.getYaml()
        self.assertHasAttr(self.modulePackage, "getYaml")
        self.modulePackage.getYaml()


class DnfModuleProfileApiTest(TestCase):
    def test_moduleProfile_getName(self):
        # ModuleProfile.getName()
        moduleProfile = libdnf.module.ModuleProfile()
        self.assertHasAttr(moduleProfile, "getName")
        moduleProfile.getName()

    def test_moduleProfile_getDescription(self):
        # ModuleProfile.getDescription()
        moduleProfile = libdnf.module.ModuleProfile()
        self.assertHasAttr(moduleProfile, "getDescription")
        moduleProfile.getDescription()

    def test_moduleProfile_getContent(self):
        # ModuleProfile.getContent()
        moduleProfile = libdnf.module.ModuleProfile()
        self.assertHasAttr(moduleProfile, "getContent")
        moduleProfile.getContent()


class DnfModuleDependenciesApiTest(TestCase):
    def test_moduleDependencies_getRequires(self):
        # ModuleDependencies.getRequires()
        moduleDependecy = libdnf.module.ModuleDependencies()
        self.assertHasAttr(moduleDependecy, "getRequires")
        moduleDependecy.getRequires()



# -*- coding: utf-8 -*-

# Copyright (C) 2012-2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

from __future__ import absolute_import
from __future__ import unicode_literals

import argparse

from dnf.cli.option_parser import OptionParser

import dnf.cli.commands
import dnf.pycomp
import dnf.util

import tests.support
from tests.support import mock


def _parse(command, args):
    parser = OptionParser()
    opts = parser.parse_main_args(args)
    opts = parser.parse_command_args(command, args)
    return parser, opts


class OptionParserTest(tests.support.TestCase):
    def setUp(self):
        self.cli = mock.Mock()
        self.command = MyTestCommand(self.cli)

    def test_parse(self):
        parser, opts = _parse(self.command, ['update', '--nogpgcheck'])
        self.assertEqual(opts.command, 'update')
        self.assertFalse(opts.gpgcheck)
        self.assertIsNone(opts.color)


class MyTestCommand(dnf.cli.commands.Command):

    aliases = ["test-cmd"]
    summary = 'summary'

    def __init__(self, cli):
        dnf.cli.commands.Command.__init__(self, cli)


class MyTestCommand2(dnf.cli.commands.Command):

    aliases = ["test-cmd2"]
    summary = 'summary2'

    def __init__(self, cli):
        dnf.cli.commands.Command.__init__(self, cli)


class OptionParserAddCmdTest(tests.support.TestCase):

    def setUp(self):
        self.cli_commands = {}
        self.parser = OptionParser()
        self.parser._ = dnf.pycomp.NullTranslations().ugettext
        self.cli = mock.Mock()

    def _register_command(self, command_cls):
        """ helper for simulate dnf.cli.cli.Cli.register_Command()"""
        for name in command_cls.aliases:
            self.cli_commands[name] = command_cls

    def test_add_commands(self):
        cmd = MyTestCommand(self.cli)
        self._register_command(cmd)
        self.parser.add_commands(self.cli_commands, "main")
        name = cmd.aliases[0]
        self.assertTrue(name in self.parser._cmd_usage)
        group, summary = self.parser._cmd_usage[name]
        self.assertEqual(group, 'main')
        self.assertEqual(summary, cmd.summary)
        self.assertEqual(self.parser._cmd_groups, set(['main']))

    def test_add_commands_only_once(self):
        cmd = MyTestCommand(self.cli)
        self._register_command(cmd)
        self.parser.add_commands(self.cli_commands, "main")
        cmd = MyTestCommand(self.cli)
        self._register_command(cmd)
        self.parser.add_commands(self.cli_commands, "plugin")
        self.assertEqual(len(self.parser._cmd_usage.keys()), 1)
        self.assertEqual(self.parser._cmd_groups, set(['main']))

    def test_cmd_groups(self):
        cmd = MyTestCommand(self.cli)
        self._register_command(cmd)
        self.parser.add_commands(self.cli_commands, "main")
        cmd = MyTestCommand2(self.cli)
        self._register_command(cmd)
        self.parser.add_commands(self.cli_commands, "plugin")
        self.assertEqual(len(self.parser._cmd_groups), 2)
        self.assertEqual(self.parser._cmd_groups, set(['main', 'plugin']))

    def test_help_option_set(self):
        opts = self.parser.parse_main_args(['-h'])
        self.assertTrue(opts.help)

    def test_help_option_notset(self):
        opts = self.parser.parse_main_args(['foo', 'bar'])
        self.assertFalse(opts.help)

    def test_get_usage(self):
        parser = argparse.ArgumentParser()
        output = [
            u'%s [options] COMMAND' % (parser.prog if parser.prog in ["dnf", "yum"] else "dnf"),
            u'',
            u'List of Main Commands:',
            u'',
            u'test-cmd                  summary',
            u'',
            u'List of Plugin Commands:',
            u'',
            u'test-cmd2                 summary2',
            u'']
        cmd = MyTestCommand(self.cli)
        self._register_command(cmd)
        self.parser.add_commands(self.cli_commands, "main")
        cmd2 = MyTestCommand2(self.cli)
        self._register_command(cmd2)
        self.parser.add_commands(self.cli_commands, "plugin")
        self.assertEqual(len(self.parser._cmd_usage.keys()), 2)
        usage = self.parser.get_usage().split('\n')
        self.assertEqual(usage, output)



# -*- coding: utf-8 -*-


from __future__ import absolute_import
from __future__ import unicode_literals

import dnf
import dnf.cli.commands

from .common import TestCase


class DnfCliCommandsApiTest(TestCase):
    def setUp(self):
        base = dnf.Base(dnf.conf.Conf())
        cli = dnf.cli.cli.Cli(base=base)
        self.command = dnf.cli.commands.Command(cli=cli)

    def test_command(self):
        # dnf.cli.commands.Command
        self.assertHasAttr(dnf.cli.commands, "Command")
        self.assertHasType(dnf.cli.commands.Command, object)

    def test_init(self):
        base = dnf.Base(dnf.conf.Conf())
        cli = dnf.cli.cli.Cli(base=base)
        _ = dnf.cli.commands.Command(cli=cli)

    def test_aliases(self):
        # dnf.cli.commands.Command.aliases
        self.assertHasAttr(self.command, "aliases")
        self.assertHasType(self.command.aliases, list)

    def test_summary(self):
        # dnf.cli.commands.Command.summary
        self.assertHasAttr(self.command, "summary")
        self.assertHasType(self.command.summary, str)

    def test_base(self):
        # dnf.cli.commands.Command.base
        self.assertHasAttr(self.command, "base")
        self.assertHasType(self.command.base, dnf.Base)

    def test_cli(self):
        # dnf.cli.commands.Command.cli
        self.assertHasAttr(self.command, "cli")
        self.assertHasType(self.command.cli, dnf.cli.cli.Cli)

    def test_pre_configure(self):
        # dnf.cli.commands.Command.pre_configure
        self.assertHasAttr(self.command, "pre_configure")
        self.command.pre_configure()

    def test_configure(self):
        # dnf.cli.commands.Command.configure
        self.assertHasAttr(self.command, "configure")
        self.command.configure()

    def test_run(self):
        # dnf.cli.commands.Command.run
        self.assertHasAttr(self.command, "run")
        self.command.run()



# -*- coding: utf-8 -*-

# Copyright (C) 2012-2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

from __future__ import absolute_import
from __future__ import unicode_literals

import dnf.match_counter

import tests.support
from tests.support import mock


class PackageStub(tests.support.MockPackage):
    @classmethod
    def several(cls, count):
        for _ in range(count):
            yield cls()

    def __init__(self, nevra='nevra-1-1.noarch', summary='summary'):
        super(PackageStub, self).__init__(nevra)
        self.summary = summary
        self.url = ''
        self.description = ''


class MatchCounterTest(tests.support.TestCase):
    def test_canonize_string_set(self):
        a = ['f', 'p']
        b = ['p']
        self.assertLess(dnf.match_counter._canonize_string_set(b, 2),
                        dnf.match_counter._canonize_string_set(a, 2))

    def test_matched(self):
        pkg = tests.support.MockPackage("humbert-1-1.noarch")
        pkg.url = url = "http://humbert.com"
        pkg.summary = summary = "Glimpses of an incomparably more poignant bliss."
        counter = dnf.match_counter.MatchCounter()
        counter.add(pkg, 'summary', 'poignant')
        counter.add(pkg, 'url', 'humbert')
        counter.add(pkg, 'summary', 'humbert')
        self.assertCountEqual(counter.matched_needles(pkg),
                              ['humbert', 'poignant'])
        self.assertCountEqual(counter.matched_keys(pkg), ['url', 'summary'])
        self.assertCountEqual(counter.matched_haystacks(pkg), [url, summary])

    def test_sorted(self):
        counter = dnf.match_counter.MatchCounter()
        self.assertEqual(counter.sorted(), [])

        counter = dnf.match_counter.MatchCounter()
        pkg1, pkg2, pkg3 = PackageStub().several(3)
        counter.add(pkg1, 'name', '')
        counter.add(pkg2, 'summary', '')
        self.assertEqual(counter.sorted(), [pkg1, pkg2])

        counter.add(pkg3, 'url', '')
        self.assertEqual(counter.sorted(), [pkg1, pkg2, pkg3])
        self.assertEqual(counter.sorted(reverse=True), [pkg1, pkg2, pkg3])

    def test_sorted_with_needles(self):
        # the same needles should be listed together:
        counter = dnf.match_counter.MatchCounter()
        pkg1, pkg2, pkg3, pkg4 = PackageStub().several(4)
        counter.add(pkg1, 'summary', 'grin')
        counter.add(pkg2, 'summary', 'foolish')
        counter.add(pkg3, 'summary', 'grin')
        counter.add(pkg4, 'summary', 'grin')

        srt = counter.sorted()
        self.assertEqual(srt.index(pkg2), 1)

        # more unique needles is more than less unique needles:
        counter = dnf.match_counter.MatchCounter()
        counter.add(pkg1, 'summary', 'a')
        counter.add(pkg1, 'summary', 'b')
        counter.add(pkg2, 'summary', 'b')
        counter.add(pkg2, 'summary', 'b')

        self.assertSequenceEqual(counter.sorted(), (pkg1, pkg2))

    def test_sorted_limit(self):
        counter = dnf.match_counter.MatchCounter()
        pkg1, pkg2, pkg3 = PackageStub().several(3)
        counter.add(pkg1, 'name', '')
        counter.add(pkg2, 'url', '')
        counter.add(pkg3, 'description', '')
        self.assertSequenceEqual(counter.sorted(limit_to=[pkg1, pkg2]),
                                 (pkg1, pkg2))

    def test_sorted_exact_match(self):
        """Exactly matching the name beats name and summary non-exact match."""
        counter = dnf.match_counter.MatchCounter()
        pkg1 = PackageStub('wednesday-1-1.noarch', 'morning')
        pkg2 = PackageStub('wednesdaymorning-1-1.noarch', "5 o'clock")
        counter.add(pkg1, 'name', 'wednesday')
        counter.add(pkg2, 'name', 'wednesday')
        counter.add(pkg2, 'summary', 'clock')
        self.assertSequenceEqual(counter.sorted(), (pkg1, pkg2))

    def test_total(self):
        counter = dnf.match_counter.MatchCounter()
        counter.add(3, 'summary', 'humbert')
        counter.add(3, 'url', 'humbert')
        counter.add(20, 'summary', 'humbert')
        self.assertEqual(len(counter), 2)
        self.assertEqual(counter.total(), 3)

    def test_distance(self):
        pkg2 = tests.support.MockPackage('rust-and-stardust-1-2.x86_64')
        pkg1 = tests.support.MockPackage('rust-1-3.x86_64')
        counter = dnf.match_counter.MatchCounter()
        counter.add(pkg1, 'name', 'rust')
        counter.add(pkg2, 'name', 'rust')
        # 'rust-and-stardust' is a worse match for 'rust' than 'rust' itself
        self.assertSequenceEqual([x.name for x in counter.sorted()],
                                 ['rust', 'rust-and-stardust'])
